﻿using DbCommon.BusinessCore.BaseCore;
using HslCommunication;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.DataCache;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Function;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using ProjectManagementSystem.Device.CommandCallback;
using ProjectManagementSystem.Device.Extenions;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Core
{
    public class PackControl
    {
        private static PackControl m_instance;
        private bool checkOnline = true;
        private bool firstStart = true;
        private bool agvHeartbeat;

        public double TimeConsumption { get; private set; }
        public ConcurrentDictionary<string, PackStation> StationDictionary { get; set; } = new ConcurrentDictionary<string, PackStation>();

        public static PackControl Instance
        {
            get
            {
                if (m_instance == null)
                {
                    m_instance = new PackControl();
                    if (!m_instance.Initialize())
                    {
                        m_instance = null;
                    }
                }
                return m_instance;
            }
        }


        private PackControl()
        {
        }


        private bool Initialize()
        {
            try
            {
                string className = AppSetting.TryGetValue<string>("PackPlcToAgv");
                if (string.IsNullOrEmpty(className))
                {
                    className = "PackPlcToAgvBase";
                }

                var assemblyName = this.GetType().Assembly.GetName().Name;
                var nameSpace = this.GetType().Namespace;
                string fullClassName = $"{nameSpace}.{className}";

                var packPlcToAgv = InstanceConstructor.GetInstance<IPackPlcToAgv>(assemblyName, fullClassName);
                if (packPlcToAgv == null)
                {
                    throw new NullReferenceException("创建的对象为null");
                }


                var configList = ExcelConfig.Instance.RouteConfigList;

                for (int i = 0; i < configList.Count; i++)
                {
                    var config = configList[i];

                    PackStation data = StationDictionary.TryGetValue(config.LocationCode, out var station) ? station : new PackStation();

                    var sClassName = config.LocationCode.GetLocationMember("PACK对接类");
                    if (!string.IsNullOrEmpty(sClassName))
                    {
                        var sPackPlcToAgv = InstanceConstructor.GetInstance<IPackPlcToAgv>(assemblyName, $"{nameSpace}.{sClassName}");
                        if (sPackPlcToAgv == null)
                        {
                            throw new NullReferenceException("创建的对象为null");
                        }

                        data.PackPlcToAgv = sPackPlcToAgv;
                    }
                    else
                    {
                        data.PackPlcToAgv = packPlcToAgv;
                    }

                    data.Id = config.LocationCode;
                    data.routeConfig = config;

                    data.请求进入导航点 = config.LocationCode.GetLocationMember("AGV请求进入导航点").ToValue<int>();

                    data.EnteringEdges = config.LocationCode.GetLocationMember("AGV正在进入路段").ToValueArray<int>();
                    data.EnteringVertexs = config.LocationCode.GetLocationMember("AGV正在进入导航点").ToValueArray<int>();

                    data.工位导航点 = config.GraphVertex;

                    data.LeavingEdges = config.LocationCode.GetLocationMember("AGV正在离开路段").ToValueArray<int>();
                    data.LeavingVertexs = config.LocationCode.GetLocationMember("AGV正在离开导航点").ToValueArray<int>();


                    data.EnteringGateEdges = config.LocationCode.GetLocationMember("AGV正在进门路段").ToValueArray<int>();
                    data.EnteringGateVertexs = config.LocationCode.GetLocationMember("AGV正在进门导航点").ToValueArray<int>();

                    data.LeavingGateEdges = config.LocationCode.GetLocationMember("AGV正在出门路段").ToValueArray<int>();
                    data.LeavingGateVertexs = config.LocationCode.GetLocationMember("AGV正在出门导航点").ToValueArray<int>();

                    data.plcEStopAddr = config.LocationCode.GetLocationMember("PLC急停");

                    StationDictionary.AddOrUpdate(data.Id, data, (key, value) => data);
                }

                Task.Factory.StartNew(UserThread, TaskCreationOptions.LongRunning);


                DeviceControl.Instance.Communication.OnInfoSkq += Communication_OnInfoSkq;
                return true;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteException("PackControl Initialize", ex);
            }

            return false;
        }

        private void TaskCustomProc()
        {
            //return;
            //AGV状态
            //0：空闲，
            //1：配送中，
            //3：完成
            //4：充电中
            //5:发生异常
            //默认值：65535
            //return;
            var agvDataList1 = Crms.PmsApi.GetAllCarrier();
            if (agvDataList1 == null) return;

            var agvDataList = checkOnline ? agvDataList1.Where(d => d.Online) : agvDataList1.Where(d => true);

            foreach (var item in StationDictionary.Values)
            {
                var config = item.routeConfig;
                if (!item.PackTaskAddCooled)
                {
                    //CLog.Instance.TaskLog.WriteInfo($"{config.LocationCode} 限制添加任务（未冷却）");
                    continue;
                }


                //已到位的AGV可以出发后任务模板
                var agvData = agvDataList?.FirstOrDefault(d => /*d.Status == 0 && */d.GraphVertex == config.GraphVertex);
                if (agvData != null && config.GraphVertex > 0)
                {
                    var taskList = Crms.PmsApi.GetTaskList();
                    if (taskList != null
                        && taskList.FirstOrDefault(d => d.Carrier == agvData.AgvID || d.BindingAGVNumber == agvData.AgvID) == null)
                    {
                        //添加任务
                        string nextLocation = GetPlcNextLocation(item, agvData.AgvID, out string templateName);

                        if (!string.IsNullOrEmpty(nextLocation)
                            && !string.IsNullOrEmpty(templateName))
                        {
                            AddPackTask(templateName, agvData.AgvID, config.LocationCode, nextLocation);
                        }
                    }
                }


                //工位前的任务导航点：可根据前任务导航点，匹配不一样的任务模板
                var vertexArray = config.LocationCode.GetLocationMember("前任务导航点").ToValueArray<int>();
                for (int index = 0; index < vertexArray.Length; index++)
                {
                    int vertex = vertexArray[index];

                    agvData = agvDataList?.FirstOrDefault(d => d.Status == 0 && d.GraphVertex > 0 && vertex == d.GraphVertex);
                    if (agvData != null)
                    {
                        string preTemplateName = config.LocationCode.GetLocationMember("前任务模板").GetArrayValue<string>(index);
                        AddPackTask(preTemplateName, agvData.AgvID, config.LocationCode);
                    }
                }


            }

        }

        private string GetPlcNextLocation(PackStation packStation, int agvId, out string templateName)
        {
            var config = packStation.routeConfig;
            if (config.LocationCode.Contains("分配"))
            {
                //模拟测试
                templateName = config.LocationCode.GetLocationMember("后任务模板");
                return config.LocationCode.GetLocationMember("下个工位").ToValueArray<string>()[0];
            }

            templateName = null;

            //检查是否有放行信号
            int releaseResult = packStation.PackPlcToAgv.ReleaseAgv(config, out string releaseMsg);
            packStation.PackReleaseMessage = releaseMsg;
            if (releaseResult <= 0)
            {
                return null;
            }
            //已有放行信号

            string nextLocation = config.LocationCode.GetLocationMember("下个工位");
            //指定工位放行
            if (releaseResult > 0 && releaseResult < 20000)
            {
                nextLocation = ExcelConfig.Instance.RouteConfigList.FirstOrDefault(d => d.StationID == releaseResult)?.LocationCode;
                if (string.IsNullOrEmpty(nextLocation))
                {
                    CLog.Instance.GetAgvLog(agvId).WriteInfo($"{config.LocationCode} 没有找到工位 指定放行活动点 {releaseResult}");
                    return null;
                }
            }

            if (string.IsNullOrEmpty(nextLocation))
            {
                CLog.Instance.GetAgvLog(agvId).WriteInfo($"{config.LocationCode} 没配置下个工位");
                return null;
            }

            templateName = config.LocationCode.GetLocationMember("后任务模板");
            nextLocation = nextLocation.ToValueArray<string>()[0];

            //检查是否有返修信号
            int allowReapairResult = packStation.PackPlcToAgv.AllowToRepair(config, out string allowReapairMsg);
            packStation.PackReleaseMessage = allowReapairMsg;
            if (allowReapairResult == -20000)
            {
                //没有配置返修地址，直接去下个工位
                CLog.Instance.GetAgvLog(agvId).WriteInfo($"{config.LocationCode} 去下个工位 {nextLocation} （{releaseMsg} {allowReapairMsg}）");
                return nextLocation;
            }

            if (allowReapairResult < 0)
            {
                return null;
            }

            if (allowReapairResult == 0)
            {
                //不需要返修去下个工位
                CLog.Instance.GetAgvLog(agvId).WriteInfo($"{config.LocationCode} 不需要返修，去下个工位 {nextLocation} （{releaseMsg} {allowReapairMsg}）");
                return nextLocation;
            }

            //已有需要返修的信号
            templateName = config.LocationCode.GetLocationMember("返修任务模板");
            if (string.IsNullOrEmpty(templateName))
            {
                CLog.Instance.GetAgvLog(agvId).WriteInfo($"{config.LocationCode} 需要返修，但没配置返修任务模板");
                return null;
            }

            //需要返修去固定返修位置
            string ngLocation = config.LocationCode.GetLocationMember("返修工位").ToValueArray<string>()[0];
            //指定工位返修放行
            if (allowReapairResult > 0 && allowReapairResult < 20000)
            {
                ngLocation = ExcelConfig.Instance.RouteConfigList.FirstOrDefault(d => d.StationID == allowReapairResult)?.LocationCode;
                if (string.IsNullOrEmpty(ngLocation))
                {
                    CLog.Instance.GetAgvLog(agvId).WriteInfo($"{config.LocationCode} 没有找到工位 指定放行返修活动点 {releaseResult}");
                    return null;
                }
            }

            if (string.IsNullOrEmpty(ngLocation))
            {
                CLog.Instance.GetAgvLog(agvId).WriteInfo($"{config.LocationCode} 需要返修，但没配置返修工位");
                return null;
            }

            CLog.Instance.GetAgvLog(agvId).WriteInfo($"{config.LocationCode} 需要返修，去返修工位 {ngLocation} （{releaseMsg} {allowReapairMsg}）");
            return ngLocation;
        }

        private bool AddPackTask(string templateName, int agvId, params string[] locations)
        {
            //添加任务
            if (string.IsNullOrEmpty(templateName)) return false;

            var taskList = Crms.PmsApi.GetTaskList();
            if (taskList == null) return false;

            var taskData = taskList.FirstOrDefault(d => d.Carrier == agvId || d.BindingAGVNumber == agvId);
            if (taskData != null)
            {
                //指定AGV已经有任务，就不添任务
                return false;
            }


            MidTaskInfoEx data = new MidTaskInfoEx();
            data.TemplateName = templateName;

            for (int i = 0; i < locations.Length; i++)
            {
                data.ParametersDic.Add($"U{i + 1}", locations[i]);
            }

            data.AgvType = "0";
            data.AgvId = agvId;
            data.Priority = 0;
            data.CreateTime = DateTime.Now;

            bool result = Crms.PmsApi.TaskAdd(data);
            if (result)
            {
                for (int i = 0; i < locations.Length; i++)
                {
                    var location = locations[i];
                    if (StationDictionary.TryGetValue(location, out var packStation))
                    {
                        packStation.LastPackTaskAddTime = DateTime.Now;
                    }
                }
            }
            return result;
        }

        private string GetAgvDataString(ResultAgvData agvData)
        {
            return agvData == null ? null : $"({agvData.AgvID}# 路段:{agvData.GraphEdge} 导航点:{agvData.GraphVertex})";
        }

        private void UpdateStationDict(bool[] forceWrite)
        {
            var agvDataList1 = Crms.PmsApi.GetAllCarrier();
            if (agvDataList1 == null) return;

            var agvDataList = checkOnline ? agvDataList1.Where(d => d.Online) : agvDataList1.Where(d => true);

            DateTime start = DateTime.Now;
            Parallel.ForEach(StationDictionary.Values, delegate (PackStation data, ParallelLoopState loopState)
            {
                try
                {
                    //心跳信号处理
                    var config = data.routeConfig;
                    if (config.PlcConnected
                        && config.PlcReadWrite != null
                        && !string.IsNullOrEmpty(config.PlcHeartbearAddr_AGV))
                    {
                        var tempResult2 = config.PlcReadWrite.Write(config.PlcHeartbearAddr_AGV, agvHeartbeat);
                        if (!tempResult2.IsSuccess)
                        {
                            CLog.Instance.SystemLog.WriteInfo($"{config.LocationCode} {config.PlcIpAddr} 写入AGV心跳：{tempResult2.IsSuccess.ToChineseString()}  {config.PlcHeartbearAddr_AGV}");
                        }
                    }

                    //计算AGV状态信号
                    var agvDataRequestEnter = agvDataList.FirstOrDefault(d => data.请求进入导航点 > 0 && d.GraphVertex == data.请求进入导航点);
                    var agvDataAtStation = agvDataList.FirstOrDefault(d => data.工位导航点 > 0 && d.GraphVertex == data.工位导航点);

                    var agvDataEntering = agvDataList.FirstOrDefault(d =>
                    (d.GraphEdge > 0 && data.EnteringEdges.Length > 0 && data.EnteringEdges.Contains(d.GraphEdge)) ||
                    (d.GraphVertex > 0 && data.EnteringVertexs.Length > 0 && data.EnteringVertexs.Contains(d.GraphVertex)));

                    var agvDataLeaving = agvDataList.FirstOrDefault(d =>
                    (d.GraphEdge > 0 && data.LeavingEdges.Length > 0 && data.LeavingEdges.Contains(d.GraphEdge)) ||
                    (d.GraphVertex > 0 && data.LeavingVertexs.Length > 0 && data.LeavingVertexs.Contains(d.GraphVertex)));

                    var agvDataEnteringGate = agvDataList.FirstOrDefault(d =>
                    (d.GraphEdge > 0 && data.EnteringGateEdges.Length > 0 && data.EnteringGateEdges.Contains(d.GraphEdge)) ||
                    (d.GraphVertex > 0 && data.EnteringGateVertexs.Length > 0 && data.EnteringGateVertexs.Contains(d.GraphVertex)));

                    var agvDataLeavingGate = agvDataList.FirstOrDefault(d =>
                    (d.GraphEdge > 0 && data.LeavingGateEdges.Length > 0 && data.LeavingGateEdges.Contains(d.GraphEdge)) ||
                    (d.GraphVertex > 0 && data.LeavingGateVertexs.Length > 0 && data.LeavingGateVertexs.Contains(d.GraphVertex)));


                    data.请求进入AGV信息 = GetAgvDataString(agvDataRequestEnter);
                    data.到位AGV信息 = GetAgvDataString(agvDataAtStation);
                    data.正在进入AGV信息 = GetAgvDataString(agvDataEntering);
                    data.正在离开AGV信息 = GetAgvDataString(agvDataLeaving);
                    data.正在进门AGV信息 = GetAgvDataString(agvDataEnteringGate);
                    data.正在出门AGV信息 = GetAgvDataString(agvDataLeavingGate);


                    data.agvDataAtStation = agvDataAtStation;
                    data.agvDataEntering = agvDataEntering;
                    data.agvDataLeaving = agvDataLeaving;


                    bool result = agvDataRequestEnter != null;
                    if (data.请求进入 != result || forceWrite[0])
                    {
                        data.请求进入 = result;
                        data.PackPlcToAgv.HandleAgvToPlc(config, "AGV请求进入", data.请求进入, data.请求进入AGV信息);
                    }

                    result = agvDataAtStation != null;
                    if (data.到位 != result || forceWrite[1])
                    {
                        data.到位 = result;
                        data.PackPlcToAgv.HandleAgvToPlc(config, "AGV到位", data.到位, data.到位AGV信息);
                    }



                    result = agvDataEntering != null;
                    if (data.正在进入 != result || firstStart)
                    {
                        data.正在进入 = result;
                        data.PackPlcToAgv.HandleAgvToPlc(config, "AGV正在进入", data.正在进入, data.正在进入AGV信息);
                    }

                    if (forceWrite[2])
                    {
                        data.正在进入 = result;
                        if (data.正在进入)
                        {
                            data.PackPlcToAgv.HandleAgvToPlc(config, "AGV正在进入", data.正在进入, data.正在进入AGV信息);
                        }
                        else
                        {
                            //清零信号需要已经离开前面的导航点
                            if (data.请求进入 == false)
                            {
                                data.PackPlcToAgv.HandleAgvToPlc(config, "AGV正在进入", data.正在进入, data.正在进入AGV信息);
                            }
                        }
                    }




                    result = agvDataLeaving != null;
                    if (data.正在离开 != result || firstStart)
                    {
                        data.正在离开 = result;
                        data.PackPlcToAgv.HandleAgvToPlc(config, "AGV正在离开", data.正在离开, data.正在离开AGV信息);
                    }

                    if (forceWrite[3])
                    {
                        data.正在离开 = result;
                        if (data.正在离开)
                        {
                            data.PackPlcToAgv.HandleAgvToPlc(config, "AGV正在离开", data.正在离开, data.正在离开AGV信息);
                        }
                        else
                        {
                            //清零信号需要已经离开前面的导航点
                            if (data.到位 == false)
                            {
                                data.PackPlcToAgv.HandleAgvToPlc(config, "AGV正在离开", data.正在离开, data.正在离开AGV信息);
                            }
                        }
                    }


                    //LogicBit 9  上升到位状态 512
                    //LogicBit 10 下降到位状态 1024
                    result = agvDataAtStation != null ? agvDataAtStation.LogicBits.GetBitVaule(9) : false;
                    data.上升到位AGV信息 = result ? $"{data.到位AGV信息} LogicBits:{agvDataAtStation.LogicBits}" : null;

                    if (data.AGV上升到位 != result || forceWrite[4])
                    {
                        data.AGV上升到位 = result;
                        data.PackPlcToAgv.HandleAgvToPlc(config, nameof(data.AGV上升到位), data.AGV上升到位, data.上升到位AGV信息);
                    }

                    result = agvDataAtStation != null ? agvDataAtStation.LogicBits.GetBitVaule(10) : false;
                    data.下降到位AGV信息 = result ? $"{data.到位AGV信息} LogicBits:{agvDataAtStation.LogicBits}" : null;

                    if (data.AGV下降到位 != result || forceWrite[5])
                    {
                        data.AGV下降到位 = result;
                        data.PackPlcToAgv.HandleAgvToPlc(config, nameof(data.AGV下降到位), data.AGV下降到位, data.下降到位AGV信息);
                    }

                    var agvId = agvDataAtStation == null ? 0 : agvDataAtStation.AgvID;
                    if (data.AGV编号 != agvId || forceWrite[1])
                    {
                        data.AGV编号 = agvId;
                        data.PackPlcToAgv.HandleAgvToPlc(config, nameof(data.AGV编号), (short)data.AGV编号, data.到位AGV信息);
                    }

                    var alarmId = agvDataAtStation == null || agvDataAtStation.AlarmList == null || agvDataAtStation.AlarmList.Count == 0 ?
                    0 : agvDataAtStation.AlarmList.OrderByDescending(d => d.alarmLevel).FirstOrDefault().alarmId;
                    if (data.AGV报警代码 != alarmId || forceWrite[1])
                    {
                        data.AGV报警代码 = alarmId;
                        data.PackPlcToAgv.HandleAgvToPlc(config, nameof(data.AGV报警代码), (short)data.AGV报警代码, data.到位AGV信息);
                    }

                    bool agvAlarm = alarmId > 0;
                    if (data.AGV报警 != agvAlarm || forceWrite[1])
                    {
                        data.AGV报警 = agvAlarm;
                        data.PackPlcToAgv.HandleAgvToPlc(config, nameof(data.AGV报警), data.AGV报警, data.到位AGV信息);
                    }




                    result = agvDataEnteringGate != null;
                    if (data.AGV正在进门 != result || firstStart || forceWrite[6])
                    {
                        data.AGV正在进门 = result;
                        data.PackPlcToAgv.HandleAgvToPlc(config, nameof(data.AGV正在进门), data.AGV正在进门, data.正在进门AGV信息);
                    }

                    result = agvDataLeavingGate != null;
                    if (data.AGV正在出门 != result || firstStart || forceWrite[7])
                    {
                        data.AGV正在出门 = result;
                        data.PackPlcToAgv.HandleAgvToPlc(config, nameof(data.AGV正在出门), data.AGV正在出门, data.正在出门AGV信息);
                    }

                    data.消息 = agvDataAtStation == null ? null : data.PackReleaseMessage;
                }
                catch (Exception ex)
                {
                    data.消息 = ex.Message;
                }

                data.耗时 = Math.Round((DateTime.Now - start).TotalMilliseconds);
            });


            //foreach (var agvData in agvDataList)
            Parallel.ForEach(agvDataList, delegate (ResultAgvData agvData, ParallelLoopState loopState)
            {
                try
                {
                    //PlcStopAgv(agvData);
                    PlcRealeseToLogicValue(agvData);
                }
                catch (Exception)
                {
                }
            });
        }

        private void PlcRealeseToLogicValue(ResultAgvData agvData)
        {
            //查找AGV所在的工位
            var station = StationDictionary.Values.FirstOrDefault(d => d.agvDataAtStation?.AgvID == agvData.AgvID);

            int idx = 13;
            string msg = null;
            bool plcRealeseAgv = false;
            bool agvLogicRealeseAgv = agvData.LogicValues?[idx] == 1;

            if (station != null)
            {
                plcRealeseAgv = station.PackPlcToAgv.ReleaseAgv(station.routeConfig, out msg) > 0;
            }
            else
            {
                msg = "AGV不在工位导航点";
            }

            if (agvLogicRealeseAgv != plcRealeseAgv)
            {
                int logicValue = plcRealeseAgv ? 1 : 0;
                var result = Crms.PmsApi.SetCarrierLogicValues(agvData.AgvID, idx, new int[] { logicValue });
                CLog.Instance.GetAgvLog(agvData.AgvID).WriteDebug($"{station?.Id} {agvData.AgvID}# LogicValues[{idx}]={logicValue} 设置逻辑值{result.ToChineseString()} （{msg}）");
            }
        }
        private void PlcStopAgv(ResultAgvData agvData)
        {
            //查找AGV所在的正在进入和正在离开的所有工位
            var stations = StationDictionary.Values.Where(d =>
            {
                if (string.IsNullOrEmpty(d.plcEStopAddr)
                || string.IsNullOrEmpty(d.routeConfig.PlcIpAddr))
                {
                    return false;
                }

                return d.agvDataEntering?.AgvID == agvData.AgvID || d.agvDataLeaving?.AgvID == agvData.AgvID;
            });

            if (stations.Count() == 0)
            {
                return;
            }

            string msg = null;
            //从AGV所在的工位中，获取需要停车的工位
            var stopStation = stations.FirstOrDefault(d =>
            {
                int result = d.PackPlcToAgv.StopAgv(d.routeConfig, out string tempMsg);
                msg += $"{d.routeConfig.LocationCode} {tempMsg}; ";

                if (result == -1 || result > 0)
                {
                    return true;
                }
                return false;
            });

            bool needPauseAgv = stopStation != null;
            bool agvPaused = agvData.AlarmList?.Exists(d => d.alarmId == 36) ?? false;

            if (needPauseAgv)
            {
                if (!agvPaused)
                {
                    bool result = Crms.PmsApi.CarrierPause(agvData.AgvID);
                    CLog.Instance.GetAgvLog(agvData.AgvID).WriteInfo($"{stopStation?.Id} {agvData.AgvID}# 暂停小车{result.ToChineseString()} （{msg}）");
                }
            }
            else
            {
                if (agvPaused)
                {
                    bool result = Crms.PmsApi.CarrierResume(agvData.AgvID);
                    CLog.Instance.GetAgvLog(agvData.AgvID).WriteInfo($"{stopStation?.Id} {agvData.AgvID}# 恢复小车{result.ToChineseString()} （{msg}）");
                }
            }
        }

        private void Communication_OnInfoSkq(InfoSkq infoSkq)
        {
            List<byte> listSend = new List<byte>();
            var bytes = infoSkq.RawBytes;

            switch (bytes[4])
            {
                case 0xE1://询问工位状态
                    {
                        listSend.Add(0XD1);
                        int mark = bytes[5];

                        if (mark == 0)
                        {
                            foreach (var station in StationDictionary.Values)
                            {
                                listSend.Add((byte)station.routeConfig.StationID);
                                listSend.Add((byte)(station.到位 ? 3 : 0));
                                listSend.Add((byte)station.AGV编号);
                                listSend.Add((byte)station.AGV报警代码);
                            }
                        }
                        else
                        {
                            var station = StationDictionary.Values.FirstOrDefault(o => o.routeConfig.StationID == mark);
                            if (station != null)
                            {
                                listSend.Add((byte)station.routeConfig.StationID);
                                listSend.Add((byte)(station.到位 ? 3 : 0));
                                listSend.Add((byte)station.AGV编号);
                                listSend.Add((byte)station.AGV报警代码);
                            }
                        }
                    }
                    break;
                case 0xE2://询问AGV状态
                    {
                        listSend.Add(0XD2);
                        int agvId = bytes[5];

                        var agv = Crms.PmsApi.GetCarrier(agvId);
                        var station = StationDictionary.Values.FirstOrDefault(o => o.AGV编号 == agvId);
                        if (agv != null && station != null)
                        {
                            listSend.Add((byte)agv.AgvID);
                            listSend.Add((byte)(station.到位 ? 3 : 0));
                            if (agv.Online)
                            {
                                listSend.Add((byte)1);
                            }
                            else
                            {
                                listSend.Add((byte)0);
                            }
                            listSend.Add((byte)station.routeConfig.StationID);
                            listSend.Add((byte)0);
                            listSend.Add((byte)0);
                            listSend.Add((byte)0);
                            listSend.Add((byte)0);
                            listSend.Add((byte)0);//速度
                            listSend.Add((byte)agv.BatteryLevel);
                        }
                    }

                    break;
                case 0xE3://控制工位状态
                    {
                        int dataLength = (int)bytes[3];
                        for (int i = 5; i < 4 + dataLength; i = i + 2)
                        {
                            var station = StationDictionary.Values.FirstOrDefault(o => o.routeConfig.StationID == bytes[i]);
                            if (station != null)
                            {
                                station.routeConfig.JData["MES放行信号"] = 20000;
                                station.routeConfig.JData["MES放行时间"] = DateTime.Now;

                                CLog.Instance.TaskLog.WriteInfo($"{station.routeConfig.LocationCode} SKQ MES放行");
                                //DeviceControl.Instance.ReleaseAgvOnStop(bytes[i], 0, station.TargetMark, station.AgvId);
                            }
                        }
                        listSend.Add(0xF3);
                        listSend.Add(0x01);
                    }
                    break;
                case 0xE4://控制AGV站点状态
                    {
                        int agvId = bytes[5];

                        var station = StationDictionary.Values.FirstOrDefault(o => o.AGV编号 == agvId && agvId != 0);
                        if (station != null)
                        {
                            int targetMark = bytes[6];

                            station.routeConfig.JData["MES放行信号"] = targetMark;
                            station.routeConfig.JData["MES放行时间"] = DateTime.Now;

                            CLog.Instance.TaskLog.WriteInfo($"{station.routeConfig.LocationCode} SKQ MES指定放行 {targetMark}");
                            //DeviceControl.Instance.ReleaseAgvOnStop(station3[i].Mark, 0, bytes[6], station3[i].AgvId);
                        }

                        listSend.Add(0XF4);
                        listSend.Add(0x01);
                    }
                    break;
                case 0xE5://控制AGV状态
                    {
                        int agvId = bytes[5];

                        int status = (int)bytes[6];
                        if (status == 2 || status == 4)
                        {
                            Crms.PmsApi.CarrierPause(agvId);
                            CLog.Instance.TaskLog.WriteInfo($"SKQ MES暂停AGV{agvId}");
                        }
                        else if (status == 1)
                        {
                            Crms.PmsApi.CarrierResume(agvId);
                            CLog.Instance.TaskLog.WriteInfo($"SKQ MES恢复AGV{agvId}");
                        }

                        listSend.Add(0XF5);
                        listSend.Add(0x01);
                    }
                    break;
                //case 0xE6://控制AGV速度
                //    {
                //        station2 = m_stationInfoDict.Values.FirstOrDefault(o => o.AgvId == bytes[5]);
                //        if (station2 != null)
                //        {
                //            int speedd = (int)bytes[6];
                //            if (speedd > 0 && speedd <= 30)
                //            {
                //                DeviceControl.Instance.RailAgv_Control1071(station2.AgvId, 0, 1);
                //            }
                //            else if (speedd > 30 && speedd <= 60)
                //            {
                //                DeviceControl.Instance.RailAgv_Control1071(station2.AgvId, 0, 2);
                //            }
                //            else if (speedd > 60)
                //            {
                //                DeviceControl.Instance.RailAgv_Control1071(station2.AgvId, 0, 3);
                //            }
                //        }
                //        listSend.Add(0XF6);
                //        listSend.Add(0x01);
                //    }
                //    break;
                case 0xE7://发送产品信息指令
                    break;
            }

            if (listSend.Count > 1)
            {
                List<byte> list2 = new List<byte>();
                list2.Add(bytes[0]);
                list2.Add(bytes[1]);
                list2.Add(bytes[2]);
                list2.Add((byte)listSend.Count);
                list2.AddRange(listSend.ToArray());
                byte[] crc = list2.ToArray().CRC16();
                list2.AddRange(crc);

                DeviceControl.Instance.Communication.SendCommand(list2.ToArray(), infoSkq.RemoteEndPoint);
            }
        }

        private void UserThread()
        {
            CLog.Instance.SystemLog.WriteDebug($"PackControl已启动");

            uint couter = 0;
            uint couter2 = 0;
            bool[] forces = new bool[10];
            while (true)
            {
                DateTime loopStartTime = DateTime.Now;

                try
                {
                    TaskCustomProc();

                    if (couter % 2 == 0)
                    {
                        //错开刷新
                        var mod = couter2 % 10;
                        if (mod >= 0 && mod <= forces.Length)
                        {
                            for (int i = 0; i < forces.Length; i++)
                            {
                                forces[i] = (mod == i);
                            }
                        }
                        couter2++;

                        //心跳信号取反
                        agvHeartbeat = !agvHeartbeat;

                        DateTime start = DateTime.Now;
                        UpdateStationDict(forces);
                        System.Diagnostics.Trace.WriteLine($"UpdateStationDict {string.Join(" ", forces.Select(d => { return d ? "1" : "0"; }))} 耗时：{(DateTime.Now - start).TotalMilliseconds:F1} ms");
                    }
                }
                catch (Exception ex)
                {
                    CLog.Instance.SystemLog.WriteException("PackControl", ex);
                    Thread.Sleep(5000);
                }

                firstStart = false;
                couter++;

                TimeConsumption = (DateTime.Now - loopStartTime).TotalMilliseconds;
                Thread.Sleep(300);
            }
        }
    }

    public class PackStation
    {
        internal string plcEStopAddr;

        internal ResultAgvData agvDataAtStation;
        internal ResultAgvData agvDataEntering;
        internal ResultAgvData agvDataLeaving;

        internal int[] EnteringVertexs;
        internal int[] EnteringEdges;
        internal int[] LeavingVertexs;
        internal int[] LeavingEdges;

        internal int[] EnteringGateVertexs;
        internal int[] EnteringGateEdges;
        internal int[] LeavingGateVertexs;
        internal int[] LeavingGateEdges;

        internal RouteConfig routeConfig { get; set; }
        internal DateTime LastPackTaskAddTime { get; set; }
        internal bool PackTaskAddCooled { get { return Math.Abs((DateTime.Now - LastPackTaskAddTime).TotalSeconds) > 5; } }
        internal IPackPlcToAgv PackPlcToAgv { get; set; }
        internal string PackReleaseMessage { get; set; }

        public string Id { get; set; }
        public int 工位导航点 { get; set; }
        public int 请求进入导航点 { get; set; }


        public bool 请求进入 { get; set; }
        public bool 正在进入 { get; set; }
        public bool 到位 { get; set; }
        public bool 正在离开 { get; set; }
        public bool 区域有Agv { get { return 正在进入 || 到位 || 正在离开; } }
        public bool AGV上升到位 { get; set; }
        public bool AGV下降到位 { get; set; }

        public bool AGV正在进门 { get; set; }
        public bool AGV正在出门 { get; set; }

        public bool AGV报警 { get; set; }
        public int AGV编号 { get; set; }
        public int AGV报警代码 { get; set; }


        public string 请求进入AGV信息 { get; set; }
        public string 正在进入AGV信息 { get; set; }
        public string 到位AGV信息 { get; set; }
        public string 正在离开AGV信息 { get; set; }
        public string 上升到位AGV信息 { get; set; }
        public string 下降到位AGV信息 { get; set; }

        public string 正在进门AGV信息 { get; set; }
        public string 正在出门AGV信息 { get; set; }

        public double 耗时 { get; set; }
        public string 消息 { get; set; }
    }

    public interface IPackPlcToAgv
    {
        /// <summary>
        /// 判断AGV是否可以放行；可以放行返回20000，放行到指定工位返回1-20000的数字（活动点）
        /// </summary>
        /// <param name="config">工位</param>
        /// <param name="msg">消息</param>
        /// <returns></returns>
        int ReleaseAgv(RouteConfig config, out string msg);
        /// <summary>
        /// 判断AGV是否需要返修；需要返修返回20000，返修到指定工位返回1-20000的数字（活动点）
        /// </summary>
        /// <param name="config">工位</param>
        /// <param name="msg">消息</param>
        /// <returns></returns>
        int AllowToRepair(RouteConfig config, out string msg);
        int StopAgv(RouteConfig config, out string msg);

        /// <summary>
        /// 将工位AGV相关的数据给线体
        /// </summary>
        /// <typeparam name="T">泛型</typeparam>
        /// <param name="config">工位</param>
        /// <param name="plcAddrDesc">地址描述</param>
        /// <param name="agvValue">数据值</param>
        /// <param name="agvInfo">AGV信息</param>
        void HandleAgvToPlc<T>(RouteConfig config, string plcAddrDesc, T agvValue, string agvInfo);
    }

    public class PackPlcToAgvBase : IPackPlcToAgv
    {
        protected int successCode = 20000;
        protected int unnecessaryCode = -20000;

        public virtual int ReleaseAgv(RouteConfig config, out string msg)
        {
            msg = null;
            if (string.IsNullOrEmpty(config.PlcIpAddr))
            {
                msg = $"没有配置IP地址(PLC)";
                return unnecessaryCode;
            }

            var plcReleaseAddr = config.LocationCode.GetLocationMember("PLC允许上升");
            if (string.IsNullOrEmpty(plcReleaseAddr))
            {
                plcReleaseAddr = config.LocationCode.GetLocationMember("PLC放行");
                if (string.IsNullOrEmpty(plcReleaseAddr))
                {
                    msg = $"没有配置放行地址";
                    return unnecessaryCode;
                }
            }

            if (!config.PlcConnected)
            {
                msg = $"PLC未连接";
                return -1;
            }

            var read = config.PlcReadWrite.ReadBool(plcReleaseAddr);
            if (!read.IsSuccess)
            {
                msg = $"读取{plcReleaseAddr}失败";
                return -1;
            }

            msg = $"读取{plcReleaseAddr} = {read.Content}";
            return read.Content ? successCode : 0;
        }

        public virtual int AllowToRepair(RouteConfig config, out string msg)
        {
            msg = null;
            if (string.IsNullOrEmpty(config.PlcIpAddr))
            {
                msg = $"没有配置IP地址(PLC)";
                return unnecessaryCode;
            }

            var plcAllowReparirAddr = config.LocationCode.GetLocationMember("PLC返修");
            if (string.IsNullOrEmpty(plcAllowReparirAddr))
            {
                msg = $"没有配置返修地址";
                return unnecessaryCode;
            }

            var read = config.PlcReadWrite.ReadBool(plcAllowReparirAddr);
            if (!read.IsSuccess)
            {
                msg = $"读取{plcAllowReparirAddr}失败";
                return -1;
            }

            msg = $"读取{plcAllowReparirAddr} = {read.Content}";
            return read.Content ? successCode : 0;
        }

        public virtual int StopAgv(RouteConfig config, out string msg)
        {
            msg = null;
            if (string.IsNullOrEmpty(config.PlcIpAddr))
            {
                msg = $"没有配置IP地址(PLC)";
                return unnecessaryCode;
            }

            var plcStopAgvAddr = config.LocationCode.GetLocationMember("PLC急停");
            if (string.IsNullOrEmpty(plcStopAgvAddr))
            {
                msg = $"没有配置急停地址";
                return unnecessaryCode;
            }

            if (!config.PlcConnected)
            {
                msg = $"PLC未连接";
                return -1;
            }

            var read = config.PlcReadWrite.ReadBool(plcStopAgvAddr);
            if (!read.IsSuccess)
            {
                msg = $"读取{plcStopAgvAddr}失败";
                return -1;
            }

            msg = $"读取{plcStopAgvAddr} = {read.Content}";
            return read.Content ? successCode : 0;
        }


        public virtual void HandleAgvToPlc<T>(RouteConfig config, string plcAddrDesc, T agvValue, string agvInfo)
        {
            if (!config.PlcConnected || config.PlcReadWrite == null) return;

            string plcAddr = config.LocationCode.GetLocationMember(plcAddrDesc);
            if (string.IsNullOrWhiteSpace(plcAddr)) return;

            var readResult = config.PlcReadWrite.ReadValue(plcAddr, agvValue, out var _);
            if (readResult) return;

            var result = config.PlcReadWrite.WriteValue(plcAddr, agvValue);
            CLog.Instance.TaskLog.WriteInfo($"{config.LocationCode} {config.PlcIpAddr} {plcAddrDesc}：{plcAddr} = {agvValue} 写入{result.IsSuccess.ToChineseString()} {agvInfo}");
        }
    }

    public class PackPlcToAgv7417 : PackPlcToAgvBase
    {
        public override int AllowToRepair(RouteConfig config, out string msg)
        {
            msg = null;
            if (string.IsNullOrEmpty(config.PlcIpAddr))
            {
                msg = $"没有配置IP地址(PLC)";
                return unnecessaryCode;
            }

            var plcAllowReparirAddr = config.LocationCode.GetLocationMember("PLC物料");
            if (string.IsNullOrEmpty(plcAllowReparirAddr))
            {
                msg = $"没有配置返修地址";
                return unnecessaryCode;
            }

            var read = config.PlcReadWrite.ReadInt16(plcAllowReparirAddr);
            if (!read.IsSuccess)
            {
                msg = $"读取{plcAllowReparirAddr}失败";
                return -1;
            }

            msg = $"读取{plcAllowReparirAddr} = {read.Content}";
            return read.Content == 22 ? successCode : 0;
        }
    }


    public class PackPlcToAgvSKQMES : PackPlcToAgvBase
    {
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            if (!config.JData.ContainsKey("MES放行信号"))
            {
                msg = $"没有MES放行信号";
                return unnecessaryCode;
            }
            if (!config.JData.ContainsKey("MES放行时间"))
            {
                msg = $"没有MES放行时间";
                return unnecessaryCode;
            }

            var result = (int)config.JData["MES放行信号"];
            var reslutTime = (DateTime)config.JData["MES放行时间"];
            if ((DateTime.Now - reslutTime).TotalSeconds > 5)
            {
                msg = $"MES放行时间超时";
                return -1;
            }

            msg = $"MES放行信号{result}";
            return result;
        }
        public override int AllowToRepair(RouteConfig config, out string msg)
        {
            msg = "不需要返修";
            return unnecessaryCode;
        }
    }

    public class PackPlcToAgvCallBox : PackPlcToAgvBase
    {
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            int[] marks = config.LocationCode.GetLocationMember("放行叫料路线").ToValueArray<int>();
            if (marks == null || marks.Length == 0)
            {
                msg = "未配置放行叫料路线";
                return unnecessaryCode;
            }

            int count = DeviceControl.Instance.Communication.InfoBoxDictionary.Values
                .Count(d => d.DataValid && marks.Contains(d.Route) && d.BoxCalled);

            if (count < marks.Length)
            {
                msg = "等待放行";
                return -1;
            }

            for (int j = 0; j < 2; j++)
            {
                for (int i = 0; i < marks.Length; i++)
                {
                    DeviceControl.Instance.CallBoxLightOff(marks[i]);
                    Thread.Sleep(100);
                }
            }

            msg = "CallBox放行";
            return successCode;
        }
        public override int AllowToRepair(RouteConfig config, out string msg)
        {
            msg = "不需要返修";
            return unnecessaryCode;
        }
    }

    public class PackPlcToAgvChargeV3 : PackPlcToAgvBase
    {
        private int arriveDelayTimeSeconds = 60;
        private LocationPropertyManager locationPropertyManager = new LocationPropertyManager();

        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            var locationData = locationPropertyManager.GetById(config.LocationCode);
            if (locationData == null)
            {
                msg = "获取库位数据失败";
                return -1;
            }

            if (!DateTime.TryParse(locationData.ModifyTime, out var modifyTime))
            {
                msg = "转换库位修改时间失败";
                return -1;
            }

            var arriveTimeSec = (DateTime.Now - modifyTime).TotalSeconds;
            if (arriveTimeSec < arriveDelayTimeSeconds)
            {
                msg = $"到达工位时间{arriveTimeSec:F0}s < {arriveDelayTimeSeconds}s";
                return -1;
            }

            //工位放行
            var baseResult = base.ReleaseAgv(config, out string baseMsg);
            if (baseResult > 0)
            {
                msg = baseMsg;
                return baseResult;
            }

            if (!PackControl.Instance.StationDictionary.TryGetValue(config.LocationCode, out var packStation))
            {
                msg = "找不到Pack工位";
                return -1;
            }

            if (packStation.agvDataAtStation == null)
            {
                msg = "工位没有小车";
                return -1;
            }

            var agvData = Crms.PmsApi.GetCarrier(packStation.agvDataAtStation.AgvID);
            if (agvData == null)
            {
                msg = "获取小车数据失败";
                return -1;
            }

            if (agvData.Status != 0)
            {
                msg = "小车不是空闲状态";
                return -1;
            }

            msg = "小车是空闲状态";
            return successCode;
        }
        public override int AllowToRepair(RouteConfig config, out string msg)
        {
            msg = "不需要返修";
            return unnecessaryCode;
        }
    }

    public class PackPlcToAgvTimeRelease : PackPlcToAgvBase
    {
        private LocationPropertyManager locationPropertyManager = new LocationPropertyManager();

        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            var locationData = locationPropertyManager.GetById(config.LocationCode);
            if (locationData == null)
            {
                msg = "获取库位数据失败";
                return -1;
            }

            if (!DateTime.TryParse(locationData.ModifyTime, out var modifyTime))
            {
                msg = "转换库位修改时间失败";
                return -1;
            }

            var arriveTimeSec = (DateTime.Now - modifyTime).TotalSeconds;
            var delayTime = config.LocationCode.GetLocationMember("定时放行").ToValue<int>();
            if (arriveTimeSec < delayTime)
            {
                msg = $"到达工位时间{arriveTimeSec:F0}s < {delayTime}s";
                return -1;
            }

            msg = $"定时放行{delayTime}s";
            return successCode;
        }
        public override int AllowToRepair(RouteConfig config, out string msg)
        {
            msg = "不需要返修";
            return unnecessaryCode;
        }
    }

    public class PackPlcToAgvAutoRelease : PackPlcToAgvBase
    {
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            msg = "自动放行";
            return successCode;
        }
        public override int AllowToRepair(RouteConfig config, out string msg)
        {
            msg = "不需要返修";
            return unnecessaryCode;
        }
    }

    public class PackPlcToAgvPadRelease : PackPlcToAgvBase
    {
        public override int ReleaseAgv(RouteConfig config, out string msg)
        {
            var dict = CacheStringToObject.ReleaseAgvDtoDict;
            if (!dict.TryGetValue(config.LocationCode, out var releaseAgvData))
            {
                msg = $"获取放行数据失败 {config.LocationCode}";
                return -1;
            }

            if (!releaseAgvData.DataValid)
            {
                msg = $"获取放行数据超时 {config.LocationCode}";
                return -1;
            }

            releaseAgvData.Status = 1;
            msg = $"获取放行数据成功 {config.LocationCode}";
            return successCode;
        }
    }
}

