﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Device.CommandCallback;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Dispatch
{
    public class ProcessTraffic : ISimpleProcess
    {
        private int[] stationIDs = ExcelConfig.Instance.RouteConfigList
            .Where(d => d.StationID > 0)
            .Select(d => d.StationID)
            .ToArray();

        public void SimpleProcess()
        {
            var dataList = ExcelConfig.Instance.TrafficList;
            if (dataList == null) return;

            for (int index = 0; index < dataList.Count; index++)
            {
                var trafficInfo = dataList[index];

                //检查管制
                trafficInfo.IsTraffic = CheckIsTraffic(trafficInfo.Marks, trafficInfo.ExtraMarks, out var infoAgv);
                trafficInfo.LastUpdateTime = DateTime.Now;

                var model = DeviceControl.Instance.Communication.InfoAgvDictionary.Values
                    .Where(d => d.DataValid && d.StatusEnum == EnumInfoAgvStatus.StopPoint && trafficInfo.Marks.Contains(d.Mark) && !stationIDs.Contains(d.Mark))
                    .OrderBy(d => d.MarkLastUpdateTime)
                    .FirstOrDefault();

                var msgTraffic = trafficInfo.IsTraffic ? $"已管制 {infoAgv?.ToSimpleString()}" : "未管制";
                var msgAgv = model != null ? $"AGV{model.AgvId}停在地标{model.Mark}" : "进管制地标无AGV";

                string msgRelease = null;
                if (!trafficInfo.IsTraffic && model != null)
                {
                    if (trafficInfo.ReleaseCold)
                    {
                        DeviceControl.Instance.ReleaseAgvOnStop(model.Mark, model.Route, model.AgvId);
                        trafficInfo.LastReleaseTime = DateTime.Now;
                        msgRelease = $"管制放行 {model.ToSimpleString()}";
                    }
                    else
                    {
                        msgRelease = "管制放行冷却中";
                    }
                }

                var msg = $"{msgTraffic} {msgAgv} {msgRelease}";
                if (trafficInfo.Message != msg)
                {
                    Log(trafficInfo, msg);
                }
                trafficInfo.Message = msg;
            }
        }

        private bool CheckIsTraffic(int[] marks, int[] extraMarks, out InfoAgv infoAgv)
        {
            var infoAgvValues = DeviceControl.Instance.Communication.InfoAgvDictionary.Values;

            infoAgv = infoAgvValues.FirstOrDefault(d => marks.Contains(d.Mark) && (d.StatusEnum != EnumInfoAgvStatus.StopPoint));
            if (infoAgv != null)
            {
                return true;
            }

            infoAgv = infoAgvValues.FirstOrDefault(d => extraMarks.Contains(d.Mark));
            return infoAgv != null;
        }

        private void Log(TrafficInfo trafficInfo, string log)
        {
            CLog.Instance.TaskLog.WriteDebug($"调度管制[{trafficInfo.MarksString}] [{trafficInfo.ExtraMarksString}] {log}");
        }
    }
}
