﻿using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Function;
using ProjectManagementSystem.Common.Logger;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Dispatch
{
    public class SimpleProcess
    {
        private int processInterval = 1000;
        private List<ISimpleProcess> taskProcesses = new List<ISimpleProcess>();


        private static SimpleProcess m_instance;
        public static SimpleProcess Instance
        {
            get
            {
                if (m_instance == null)
                {
                    m_instance = new SimpleProcess();
                    if (!m_instance.Initialize())
                    {
                        m_instance = null;
                    }
                }
                return m_instance;
            }
        }

        private SimpleProcess()
        {
        }

        public bool Initialize()
        {
            try
            {
                var assemblyName = this.GetType().Assembly.GetName().Name;
                var nameSpace = this.GetType().Namespace;

                var processArray = Common.Function.AppSetting.TryGetValue<string>("TaskProcess").ToValueArray<string>();
                foreach (var item in processArray)
                {

                    string fullClassName = $"{nameSpace}.{item}";
                    var processObj = Common.Function.InstanceConstructor.GetInstance<ISimpleProcess>(assemblyName, fullClassName);

                    if (processObj != null)
                    {
                        taskProcesses.Add(processObj);
                    }
                }

                var interval = AppSetting.TryGetValue<int>("TaskProcessInterval");
                if (interval > 0)
                {
                    processInterval = interval;
                }

                if (taskProcesses.Count > 0)
                {
                    Task.Factory.StartNew(CustomThread, TaskCreationOptions.LongRunning);
                }

                return true;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteException("SimpleProcess Initialize", ex);
            }

            return false;
        }

        private void CustomThread()
        {
            Thread.Sleep(2000);
            CLog.Instance.SystemLog.WriteDebug($"CustomProc已启动");
            while (true)
            {
                try
                {
                    for (int i = 0; i < taskProcesses.Count; i++)
                    {
                        taskProcesses[i].SimpleProcess();
                    }
                }
                catch (Exception ex)
                {
                    CLog.Instance.TaskLog.WriteException("CustomProc", ex);
                    Thread.Sleep(5000);
                }
                Thread.Sleep(processInterval);
            }
        }
    }


    public interface ISimpleProcess
    {
        void SimpleProcess();
    }
}
