﻿using ProjectManagementSystem.Common.Logger;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Function
{
    public class AppSetting
    {
        private static string m_configurationFilePath;
        private static Configuration m_configuration;

        private static string hostIp;
        private static int hostPort;

        static AppSetting()
        {
            m_configurationFilePath = System.Reflection.Assembly.GetEntryAssembly().Location;
            m_configuration = ConfigurationManager.OpenExeConfiguration(m_configurationFilePath);

            hostIp = GetWebHostIp();
            hostPort = GetWebHostPort();
        }

        public static string GetWebHostIp()
        {
            string key = "WebHostIp";
            string address = TryGetValue<string>(key);
            if (string.IsNullOrEmpty(address))
            {
                address = "localhost";
                AddOrUpdate(key, address);
            }
            return address;
        }

        public static int GetWebHostPort()
        {
            string key = "WebHostPort";
            int value = TryGetValue<int>(key);
            if (value <= 0)
            {
                value = 63300;
                AddOrUpdate(key, value.ToString());
            }
            return value;
        }

        private static void AddOrUpdate(string key, string value)
        {
            if (!m_configuration.AppSettings.Settings.AllKeys.Contains(key))
            {
                m_configuration.AppSettings.Settings.Add(new KeyValueConfigurationElement(key, value));
                CLog.Instance.SystemLog.WriteDebug($"添加配置\"{key}\"={value}");
            }
            else
            {
                m_configuration.AppSettings.Settings[key].Value = value;
                CLog.Instance.SystemLog.WriteDebug($"更新配置\"{key}\"={value}");
            }
        }

        public static string GetCasunAgvUrl()
        {
            return $"http://{hostIp}:9528";
        }

        public static string GetAutoUpdateUrl()
        {
            return $"http://{hostIp}:{hostPort}/api/ver/pmsxml";
        }

        public static string GetPmsApiUrl(string apiName)
        {
            return $"http://{hostIp}:62200/api/agv/{apiName}";
        }
        public static string GetCrmsApiUrl(string routes)
        {
            return $"http://{hostIp}:8100/api/{routes}";
        }

        public static string GetPmsApiCustomUrl(string apiName)
        {
            return $"http://{hostIp}:{hostPort}/api/pms/{apiName}";
        }
        public static string GetAgvsWebUrl(string routes)
        {
            return $"http://{hostIp}:8011/api/{routes}";
        }

        public static T TryGetValue<T>(string key)
        {
            T result;
            try
            {
                if (!m_configuration.AppSettings.Settings.AllKeys.Contains(key))
                {
                    result = default(T);
                    CLog.Instance.SystemLog.WriteDebug($"读取配置不包含 \"{key}\"， 返回默认值:{result}");
                    return result;
                }

                string value = m_configuration.AppSettings.Settings[key].Value;

                if (string.IsNullOrEmpty(value)) return default(T);
                return (T)Convert.ChangeType(value, typeof(T));

                //result = ((!string.IsNullOrEmpty(value)) ? parseFunc(value) : defaultTValueFunc());
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteException("AppSetting", ex);
                result = default(T);
            }
            return result;
        }

        public static bool TrySaveValue(string key, string value)
        {
            try
            {
                if (!m_configuration.AppSettings.Settings.AllKeys.Contains(key))
                {
                    m_configuration.AppSettings.Settings.Add(key, value);
                }
                else
                {
                    m_configuration.AppSettings.Settings[key].Value = value;
                }

                m_configuration.Save(ConfigurationSaveMode.Modified);
                ConfigurationManager.RefreshSection("appSettings");

                CLog.Instance.SystemLog.WriteDebug($"保存配置\"{key}\"={value}");
                return true;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteException("AppSetting", ex);
            }
            return false;
        }

        public static bool TrySaveValues(IDictionary<string, string> keyValues)
        {
            try
            {
                foreach (var item in keyValues)
                {
                    var key = item.Key;
                    var value = item.Value;

                    if (!m_configuration.AppSettings.Settings.AllKeys.Contains(key))
                    {
                        m_configuration.AppSettings.Settings.Add(key, value);
                    }
                    else
                    {
                        m_configuration.AppSettings.Settings[key].Value = value;
                    }

                    CLog.Instance.SystemLog.WriteDebug($"保存配置\"{key}\"={value}");
                }

                m_configuration.Save(ConfigurationSaveMode.Modified);
                ConfigurationManager.RefreshSection("appSettings");

                return true;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteException("AppSetting", ex);
            }
            return false;
        }

        public static KeyValueConfigurationCollection GetSettingsCollection()
        {
            return m_configuration.AppSettings.Settings;
        }
    }
}
