﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using Newtonsoft.Json;
using ProjectManagementSystem.Common.Function;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Service
{
    public class CustomerApi
    {
        private static string url = AppSetting.TryGetValue<string>("CustomerAddress");
        private static RestApi restApi = new RestApi();

        public static bool ReportAgvARrrived(string taskId, string station, string type, out string content)
        {
            var p = new { TaskId = taskId, Pos = station };
            return CallWebService<bool>("ReportAgvARrrived", p, d => d, out content);
        }

        public static bool AgvAlarm(string msg, string agvNo, string MESNo, out string content)
        {
            var body = new { msg = msg, agvNo = agvNo, MESNo = MESNo };
            return Post<ResponseResult>("AgvAlarm", body, d => d.code == 200 && d.success, out content);
        }

        private static string GetApiUrl(string apiName)
        {
            string apiUrl = AppSetting.TryGetValue<string>($"{apiName}Url");
            if (string.IsNullOrEmpty(apiUrl))
            {
                apiUrl = $@"{url}{apiName}";
            }

            return apiUrl;
        }

        public static bool Post<T>(string apiName, object body, Predicate<T> predicate, out string content)
        {
            content = null;
            bool result = false;

            try
            {
                string apiUrl = GetApiUrl(apiName);
                var res = restApi.Post<object>(apiUrl, null, body, out content);
                CLog.Instance.SystemLog.WriteDebug($"Post {apiUrl}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");

                var data = JsonConvert.DeserializeObject<T>(content);
                result = predicate(data);
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);

                content = ex.Message;
            }

            return result;
        }

        public static bool CallWebService<T>(string apiName, object pars, Predicate<T> predicate, out string content)
        {
            content = null;
            bool result = false;

            try
            {
                string apiUrl = GetApiUrl(apiName);
                var res = restApi.Get<T>(apiUrl, pars, null, out content);
                CLog.Instance.SystemLog.WriteDebug($"Get {apiUrl}{Environment.NewLine}  {JsonConvert.SerializeObject(pars)}{Environment.NewLine}  {content}");

                var data = XmlTools.XmlDeserialize<T>(content);
                result = predicate(data);
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);

                content = ex.Message;
            }

            return result;
        }
    }



    public class CustomerMesApi
    {
        private static string url = AppSetting.TryGetValue<string>("CustomerAddress");
        private static RestApi restApi2 = new RestApi();

        private static string GetApiUrl(string apiName)
        {
            string apiUrl = AppSetting.TryGetValue<string>($"{apiName}Url");
            if (string.IsNullOrEmpty(apiUrl))
            {
                apiUrl = $@"{url}{apiName}";
            }

            return apiUrl;
        }

        public static bool AgvChargePileData(object body, out string content)
        {
            string apiName = "AgvChargePileData";
            var result = MesApi(apiName, body, out content);

            return result;
        }

        public static bool AgvAlarmData(object body, out string content)
        {
            string apiName = "AgvAlarmData";
            var result = MesApi(apiName, body, out content);

            return result;
        }

        public static bool AgvStatusData(object body, out string content)
        {
            string apiName = "AgvStatusData";
            var result = MesApi(apiName, body, out content);

            return result;
        }

        private static bool MesApi(string apiName, object body, out string content)
        {
            content = null;
            bool result = false;

            try
            {
                string apiUrl = GetApiUrl(apiName);

                var res = restApi2.Post<ResponseResult2>(apiUrl, null, body, out content);
                result = res.resultCode == "200";

                if (!result)
                {
                    CLog.Instance.SystemLog.WriteDebug($"Post {apiUrl}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");
                }
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);

                content = ex.Message;
            }

            return result;
        }
    }
}
