﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using Mapster;
using Newtonsoft.Json.Linq;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.DataCache;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectManagementSystem.Language;

namespace ProjectManagementSystem.Common.Service
{
    public class PmsTaskService
    {
        private static PmsTaskService m_instance;
        protected static readonly object locker = new object();

        private LocationPropertyManager locationManager = new LocationPropertyManager();
        private PmsTaskInfoManager pmsTaskInfoManger = new PmsTaskInfoManager();
        private AreaTrafficManager areaTrafficManager = new AreaTrafficManager();
        private CallTaskManager callTaskManager = new CallTaskManager();

        public static PmsTaskService Instance
        {
            get
            {
                if (m_instance == null)
                {
                    lock (locker)
                    {
                        if (m_instance == null)
                        {
                            m_instance = new PmsTaskService();
                        }
                    }
                }
                return m_instance;
            }
        }
        public bool TaskAdd(MidTaskInfoEx data, string[] posArray = null)
        {
            lock (locker)
            {
                if (CheckLocaitonStatus(data, posArray))
                {
                    if (LockLocationStatus(data, posArray))
                    {
                        bool result = Crms.PmsApi.TaskAdd(data);
                        if (!result)
                        {
                            RecoverLocationStatus(data, posArray);
                        }
                        return result;

                    }
                    else
                    {
                        RecoverLocationStatus(data, posArray);
                        return false;
                    }
                }

                return false;
            }
        }
        public WebMgrResponse TaskAdd(PmsTaskInfoDto data)
        {
            lock (locker)
            {
                WebMgrResponse webMgrResponse = PmsTaskInfoCheck(data);
                if (webMgrResponse.code != 20000)
                {
                    return webMgrResponse;
                }

                return PmsTaskAdd(data);
            }
        }
        public WebMgrResponse TaskAddNoCheck(PmsTaskInfoDto data, string[] posArray = null)
        {
            lock (locker)
            {
                data.WareHouseCode = ExcelConfig.Instance.GetRouteConfig(data.LocationCode)?.WareHouseCode;
                data.TargetWareHouseCode = ExcelConfig.Instance.GetRouteConfig(data.TargetLocationCode)?.WareHouseCode;
                return PmsTaskAdd(data, posArray);
            }
        }
        public WebMgrResponse TaskAppointment(PmsTaskInfoDto data)
        {
            var webMgrResponse = new WebMgrResponse();

            var list = pmsTaskInfoManger.GetList();
            int count = list
                .Where(d => string.IsNullOrEmpty(d.LocationCode) && string.IsNullOrEmpty(d.TargetLocationCode) ? d.WareHouseCode == data.WareHouseCode && d.TargetWareHouseCode == data.TargetWareHouseCode : true)
                //.Where(d => string.IsNullOrEmpty(d.WareHouseCode) ? true : d.WareHouseCode == data.WareHouseCode)
                .Where(d => string.IsNullOrEmpty(d.LocationCode) ? true : d.LocationCode == data.LocationCode)
                //.Where(d => string.IsNullOrEmpty(d.TargetWareHouseCode) ? true : d.TargetWareHouseCode == data.TargetWareHouseCode)
                .Where(d => string.IsNullOrEmpty(d.TargetLocationCode) ? true : d.TargetLocationCode == data.TargetLocationCode)
                .Count();

            if (count > 0)
            {
                webMgrResponse.message = $"{Langs.TaskAppointmentExists} {data.LocationCode}{data.TargetLocationCode}";
                return webMgrResponse;
            }

            if (string.IsNullOrEmpty(data.TaskID))
            {
                data.TaskID = DateTime.Now.ToString("yyyyMMddHHmmssfffffff");
            }

            var destData = data.Adapt<PmsTaskInfo>();
            destData.CreateTime = DateTime.Now;
            bool result = pmsTaskInfoManger.Insert(destData);
            if (result)
            {
                webMgrResponse.code = 20000;
                webMgrResponse.data = data;
                webMgrResponse.message = Langs.TaskAppointmentSuccessful;
            }

            CLog.Instance.TaskLog.WriteInfo($"{data.TaskID} ({data.WareHouseCode} {data.LocationCode} {data.MaterialId}) => ({data.TargetWareHouseCode} {data.TargetLocationCode}) {Langs.TaskAppointment}{result.ToChineseString()}");
            return webMgrResponse;
        }
        public WebMgrResponse TaskTransport(RequestTaskTransport taskInfo)
        {
            PmsTaskInfoDto pmsTaskInfo = new PmsTaskInfoDto();
            pmsTaskInfo.TaskID = taskInfo.TaskId;
            pmsTaskInfo.LocationCode = taskInfo.StartPos;
            pmsTaskInfo.TargetLocationCode = taskInfo.EndPos;

            var webMgrResponse = TaskAdd(pmsTaskInfo);
            bool result = webMgrResponse?.code == 20000;

            return webMgrResponse;
        }
        public WebMgrResponse TaskCancel(RequestTaskCancel taskInfo)
        {
            var webMgrResponse = new WebMgrResponse();

            var result = Crms.PmsApi.TaskCancel(taskInfo.TaskID);

            webMgrResponse.code = result ? 20000 : 9999;
            webMgrResponse.message = result.ToChineseString();

            return webMgrResponse;
        }


        public WebMgrResponse ReleaseAgv(ReleaseAgvDto data)
        {
            data.LastUpdateTime = DateTime.Now;
            var dict = CacheStringToObject.ReleaseAgvDtoDict;

            dict.AddOrUpdate(data.LocationCode, data, (key, value) => data);

            bool result = false;
            for (int i = 0; i < 10; i++)
            {
                if (dict[data.LocationCode].Status > 0)
                {
                    result = dict.TryRemove(data.LocationCode, out ReleaseAgvDto outData);
                    break;
                }
                System.Threading.Thread.Sleep(100);
            }

            WebMgrResponse res = new WebMgrResponse();
            if (!result)
            {
                res.message = $"{Langs.ReleaseFailed} {data.LocationCode}";
                return res;
            }

            res.code = 20000;
            res.message = $"{Langs.ReleaseSuccessful} {data.LocationCode}";
            res.data = data;

            return res;
        }
        public virtual WebMgrResponse GetLocationProperty(LocationPropertyQueryDto data)
        {
            WebMgrResponse webMgrResponse = new WebMgrResponse();

            //查询信息
            var locationDataList = locationManager.QueryDataList(data.WareHouseCode, data.Area, data.LocationCode);

            var dtoList = locationDataList.Adapt<List<LocationPropertyDto>>();
            for (int i = 0; i < dtoList.Count; i++)
            {
                var dto = dtoList[i];

                dto.TargetWareHouseCode = dto.LocationCode.GetLocationMember("目标仓库");
                dto.SourceWareHouseCode = dto.LocationCode.GetLocationMember("源仓库");
            }

            webMgrResponse.code = 20000;
            //webMgrResponse.data = Newtonsoft.Json.JsonConvert.SerializeObject(dtoList);
            webMgrResponse.data = dtoList;
            webMgrResponse.message = Langs.Successful;
            return webMgrResponse;
        }
        public virtual WebMgrResponse UpdateLocationProperty(LocationPropertyUpdateDto data)
        {
            lock (locker)
            {
                WebMgrResponse webMgrResponse = new WebMgrResponse();

                //查询信息
                var locationData = locationManager.GetById(data.LocationCode);
                if (locationData == null)
                {
                    webMgrResponse.message = $"{data.LocationCode} {Langs.LocationGetInfoFailed}";
                    return webMgrResponse;
                }

                if (!string.IsNullOrEmpty(locationData.CurrentTaskID))
                {
                    if (LocationHasStepTask(data.LocationCode, out string taskId))
                    {
                        webMgrResponse.message = $"{data.LocationCode} {Langs.LocationOccupiedByTask} {taskId}";
                        return webMgrResponse;
                    }
                }

                string tempTaskId = locationData.CurrentTaskID;

                locationData.Status = data.Status;
                locationData.MaterialId = data.MaterialId;
                locationData.CurrentTaskID = string.Empty;
                bool result = locationManager.UpdateWithTaskId(locationData, tempTaskId);

                webMgrResponse.message = $"{data.LocationCode} {Langs.LocationStatusChange} {data.Status} {data.MaterialId} {result.ToChineseString()}";
                CLog.Instance.SystemLog.WirteInfoCaller(webMgrResponse.message);

                if (result)
                {
                    webMgrResponse.code = 20000;
                }

                return webMgrResponse;
            }
        }

        public WebMgrResponse AreaTrafficIn(AreaTrafficDto data)
        {
            lock (locker)
            {
                WebMgrResponse res = new WebMgrResponse();

                var tData = areaTrafficManager.GetById(data.Id);
                if (tData == null)
                {
                    AreaTraffic traffic = data.Adapt<AreaTraffic>();
                    traffic.LastInTime = DateTime.Now;

                    var result = areaTrafficManager.Insert(traffic);
                    if (!result)
                    {
                        res.message = $"创建管制区失败 Id:{data.Id}";
                        return res;
                    }

                    res.code = result ? 20000 : 9999;
                    res.message = $"创建管制区{result.ToChineseString()} Id:{data.Id}";
                }
                else
                {
                    var tDataText = $"Id:{tData.Id} FactoryId:{tData.FactoryId} AgvId:{tData.AgvId}";

                    if (data.FactoryId == tData.FactoryId && data.AgvId == tData.AgvId)
                    {
                        res.code = 20000;
                        res.message = $"进管制区重复申请 {tDataText}";
                        return res;
                    }

                    if (!string.IsNullOrEmpty(tData.FactoryId)
                        || !string.IsNullOrEmpty(tData.AgvId))
                    {
                        res.message = $"管制区已被占用 {tDataText}";
                        return res;
                    }

                    tData.FactoryId = data.FactoryId;
                    tData.AgvId = data.AgvId;
                    tData.LastInTime = DateTime.Now;
                    var updateResult = areaTrafficManager.Update(tData);

                    res.code = updateResult ? 20000 : 9999;
                    res.message = $"进管制区{updateResult.ToChineseString()} Id:{tData.Id}";
                }

                return res;
            }
        }
        public WebMgrResponse AreaTrafficOut(AreaTrafficDto data)
        {
            lock (locker)
            {
                WebMgrResponse res = new WebMgrResponse();

                var tData = areaTrafficManager.GetById(data.Id);
                if (tData == null)
                {
                    res.message = $"获取管制区信息失败 Id:{data.Id}";
                    return res;
                }

                var tDataText = $"Id:{tData.Id} FactoryId:{tData.FactoryId} AgvId:{tData.AgvId}";

                if (string.IsNullOrEmpty(tData.FactoryId) && string.IsNullOrEmpty(tData.AgvId))
                {
                    res.code = 20000;
                    res.message = $"该车已离开管制区 且管制区未被占用 {tDataText}";
                    return res;
                }

                if (data.FactoryId != tData.FactoryId
                    || data.AgvId != tData.AgvId)
                {
                    res.code = 20000;
                    res.message = $"该车已离开管制区 且管制区已被占用 {tDataText}";
                    return res;
                }

                tData.FactoryId = string.Empty;
                tData.AgvId = string.Empty;
                tData.LastOutTime = DateTime.Now;
                var updateResult = areaTrafficManager.Update(tData);

                res.code = updateResult ? 20000 : 9999;
                res.message = $"出管制区{updateResult.ToChineseString()} Id:{tData.Id}";

                return res;
            }
        }

        public WebMgrResponse CallTaskAdd(RequestCallTask data)
        {
            lock (locker)
            {
                var task = new CallTask();
                task.TaskID = string.IsNullOrWhiteSpace(data.TaskID) ? DateTime.Now.ToString("yyyyMMddHHmmssfffffff") : data.TaskID;
                task.TemplateName = data.TemplateName;
                task.CallTime = DateTime.Now;

                task.AllWarehouse = data.AllWarehouse;

                var result = callTaskManager.Insert(task);

                WebMgrResponse res = new WebMgrResponse();
                res.code = result ? 20000 : 9999;
                res.data = null;
                res.message = $"添加任务{result.ToChineseString()}，任务号{task.TaskID}";

                return res;
            }
        }
        public WebMgrResponse CallTaskCancel(RequestTaskCancel taskInfo)
        {
            WebMgrResponse res = new WebMgrResponse();

            var data = callTaskManager.GetById(taskInfo.TaskID);
            if (data == null)
            {
                res.message = $"查询任务ID失败，{taskInfo.TaskID}";
                return res;
            }

            data.TaskStatus = EnumTaskStatus.任务手动删除;
            var result = callTaskManager.Update(data);

            var log = $"删除任务{result.ToChineseString()}，任务号{taskInfo.TaskID}";
            CLog.Instance.TaskLog.WriteInfo(log);

            res.code = result ? 20000 : 9999;
            res.message = log;
            return res;
        }
        public WebMgrResponse CallTaskGetList()
        {
            var dataList = callTaskManager.GetList(d => d.TaskStatus < EnumTaskStatus.任务完成);

            var dataList2 = dataList.Select(d => new { d.TaskID, d.TemplateName, d.AllWarehouse, d.Carrier }).ToList();

            WebMgrResponse res = new WebMgrResponse();
            res.code = 20000;
            res.data = dataList2;
            res.message = "成功";
            return res;
        }

        protected virtual WebMgrResponse PmsTaskInfoCheck(PmsTaskInfoDto pmsTaskInfo)
        {
            var webMgrResponse = new WebMgrResponse();

            //匹配取货点
            if (string.IsNullOrEmpty(pmsTaskInfo.LocationCode))
            {
                if (string.IsNullOrEmpty(pmsTaskInfo.WareHouseCode))
                {
                    webMgrResponse.message = Langs.SourceWarehouseAndSourceLocationAllEmpty;
                    return webMgrResponse;
                }

                string location = locationManager.QueryLocation_Out(pmsTaskInfo.WareHouseCode, LocationStatus.Filled, pmsTaskInfo.MaterialId);
                if (string.IsNullOrEmpty(location))
                {
                    webMgrResponse.message = $"{Langs.SourceLocationMatchFailed} {Langs.TipWareHouseCode}{pmsTaskInfo.WareHouseCode} {Langs.TipMaterialId}{pmsTaskInfo.MaterialId}";
                    return webMgrResponse;
                }
                pmsTaskInfo.LocationCode = location;
            }

            //查询取货点信息
            var uLocationData = locationManager.GetById(pmsTaskInfo.LocationCode);
            if (uLocationData == null)
            {
                webMgrResponse.message = $"{Langs.SourceLocationGetInfoFailed} {pmsTaskInfo.LocationCode}";
                return webMgrResponse;
            }
            if (uLocationData.Status != LocationStatus.Filled)
            {
                webMgrResponse.message = $"{Langs.SourceLocationMustBeFilled} {uLocationData.LocationCode}";
                return webMgrResponse;
            }
            if (!string.IsNullOrEmpty(uLocationData.CurrentTaskID))
            {
                webMgrResponse.message = $"{Langs.SourceLocationOccupiedByTask} {uLocationData.LocationCode} {uLocationData.CurrentTaskID}";
                return webMgrResponse;
            }
            if (!locationManager.CheckLocation(uLocationData, LocationStatus.Empty))
            {
                webMgrResponse.message = $"{Langs.SourceLocationObstacleDetectionFailed} {uLocationData.LocationCode}";
                return webMgrResponse;
            }
            if (!locationManager.CheckLocationColumnNoTask(uLocationData, out var taskLocation1))
            {
                webMgrResponse.message = $"{Langs.SourceLocationHaveColumnTask} {uLocationData.LocationCode} （{taskLocation1?.LocationCode}{Langs.TaskID}{taskLocation1?.CurrentTaskID}）";
                return webMgrResponse;
            }


            //匹配放货点
            if (string.IsNullOrEmpty(pmsTaskInfo.TargetLocationCode))
            {
                if (string.IsNullOrEmpty(pmsTaskInfo.TargetWareHouseCode))
                {
                    webMgrResponse.message = Langs.TargetWarehouseAndTargetLocationAllEmpty;
                    return webMgrResponse;
                }


                string location = locationManager.QueryLocation_In(pmsTaskInfo.TargetWareHouseCode);
                if (string.IsNullOrEmpty(location))
                {
                    webMgrResponse.message = $"{Langs.TargetLocationMatchFailed} {Langs.TipWareHouseCode}{pmsTaskInfo.TargetWareHouseCode}";
                    return webMgrResponse;
                }
                pmsTaskInfo.TargetLocationCode = location;
            }

            //查询放货点信息
            var dLocationData = locationManager.GetById(pmsTaskInfo.TargetLocationCode);
            if (dLocationData == null)
            {
                webMgrResponse.message = $"{Langs.TargetLocationGetInfoFailed} {pmsTaskInfo.TargetLocationCode}";
                return webMgrResponse;
            }
            if (dLocationData.Status != LocationStatus.Empty)
            {
                webMgrResponse.message = $"{Langs.TargetLocationMustBeEmpty} {dLocationData.LocationCode}";
                return webMgrResponse;
            }
            if (!string.IsNullOrEmpty(dLocationData.CurrentTaskID))
            {
                webMgrResponse.message = $"{Langs.TargetLocationOccupiedByTask} {dLocationData.LocationCode} {dLocationData.CurrentTaskID}";
                return webMgrResponse;
            }
            if (!locationManager.CheckLocation(dLocationData, LocationStatus.Empty))
            {
                webMgrResponse.message = $"{Langs.TargetLocationObstacleDetectionFailed} {dLocationData.LocationCode}";
                return webMgrResponse;
            }
            if (!locationManager.CheckLocationColumnNoTask(dLocationData, out var taskLocation2))
            {
                webMgrResponse.message = $"{Langs.TargetLocationHaveColumnTask} {dLocationData.LocationCode} （{taskLocation2?.LocationCode}{Langs.TaskID}{taskLocation2?.CurrentTaskID}）";
                return webMgrResponse;
            }



            pmsTaskInfo.WareHouseCode = uLocationData.WareHouseCode;
            pmsTaskInfo.TargetWareHouseCode = dLocationData.WareHouseCode;

            webMgrResponse.code = 20000;
            webMgrResponse.data = pmsTaskInfo;
            return webMgrResponse;
        }
        private WebMgrResponse PmsTaskAdd(PmsTaskInfoDto pmsTaskInfo, string[] posArray = null)
        {
            MidTaskInfoEx data = new MidTaskInfoEx();
            data.TemplateName = string.IsNullOrEmpty(pmsTaskInfo.TemplateName) ? "BOOK2" : pmsTaskInfo.TemplateName;
            data.ParametersDic.Add("U1", pmsTaskInfo.LocationCode);
            data.ParametersDic.Add("U2", pmsTaskInfo.TargetLocationCode);
            data.ParentTaskId = string.IsNullOrEmpty(pmsTaskInfo.ParentTaskId) ? GetParentTaskID(pmsTaskInfo) : pmsTaskInfo.ParentTaskId;//查找取货点有没有可以续接的任务
            data.AgvId = pmsTaskInfo.Carrier;
            data.AgvType = string.IsNullOrEmpty(pmsTaskInfo.CarrierType) ? data.AgvType : pmsTaskInfo.CarrierType;
            data.ClusterNo = pmsTaskInfo.ClusterNo;
            data.Priority = pmsTaskInfo.Priority;
            data.CreateTime = DateTime.Now;

            if (!string.IsNullOrEmpty(pmsTaskInfo.TaskID))
            {
                data.TaskId = pmsTaskInfo.TaskID;
            }

            TaskMatch(pmsTaskInfo, data);

            if (!LockLocationStatus(data, posArray))
            {
                RecoverLocationStatus(data, posArray);
                WebMgrResponse webMgr = new WebMgrResponse();
                webMgr.message = Langs.LocationLockFailed;
                return webMgr;
            }

            var webMgrResponse = Crms.PmsApi.TaskAddResponse(data);
            bool result = webMgrResponse?.code == 20000;
            if (!result)
            {
                RecoverLocationStatus(data, posArray);
            }

            CLog.Instance.TaskLog.WriteInfo($"{data.TaskId} {data.ParentTaskId} {data.TemplateName} {data.Parameters} {webMgrResponse.message}");
            return webMgrResponse;
        }
        private void TaskMatch(PmsTaskInfoDto pmsTaskInfo, MidTaskInfoEx data)
        {
            //匹配任务模板
            var tempMatchList = ExcelConfig.Instance.TaskMatchList;
            if (tempMatchList != null
                && tempMatchList.Count > 0)
            {
                var tempMatch = tempMatchList
                    .Where(d => d.LocationCode == pmsTaskInfo.LocationCode && d.TargetLocationCode == pmsTaskInfo.TargetLocationCode)
                    .FirstOrDefault();

                if (tempMatch == null)
                {
                    tempMatch = tempMatchList
                    .Where(d => string.IsNullOrEmpty(d.WarehouseCode) ? true : d.WarehouseCode == pmsTaskInfo.WareHouseCode)
                    .Where(d => string.IsNullOrEmpty(d.LocationCode) ? true : d.LocationCode == pmsTaskInfo.LocationCode)
                    .Where(d => string.IsNullOrEmpty(d.TargetWarehouseCode) ? true : d.TargetWarehouseCode == pmsTaskInfo.TargetWareHouseCode)
                    .Where(d => string.IsNullOrEmpty(d.TargetLocationCode) ? true : d.TargetLocationCode == pmsTaskInfo.TargetLocationCode)
                    .FirstOrDefault();
                }

                if (tempMatch != null)
                {
                    data.TemplateName = tempMatch.TemplateName;
                    data.AgvId = tempMatch.AgvId > 0 ? tempMatch.AgvId : data.AgvId;
                    data.AgvType = !string.IsNullOrEmpty(tempMatch.AgvType) ? tempMatch.AgvType : data.AgvType;
                    data.ClusterNo = !string.IsNullOrEmpty(tempMatch.ClusterNo) ? tempMatch.ClusterNo : data.ClusterNo;
                    data.Priority = tempMatch.Priority > 0 ? tempMatch.Priority : data.Priority;
                    CLog.Instance.TaskLog.WriteInfo($"{data.TaskId} {tempMatch.WarehouseCode} {tempMatch.LocationCode} => {tempMatch.TargetWarehouseCode} {tempMatch.TargetLocationCode}  " +
                        $"{Langs.MatchTaskInfo} {tempMatch.TemplateName} AgvId:{tempMatch.AgvId} AgvType:{tempMatch.AgvType} Priority:{tempMatch.Priority}");
                }
            }
        }
        private WebMgrResponse CheckParentTaskID(PmsTaskInfoDto data)
        {
            var webMgrResponse = new WebMgrResponse();

            if (!data.LocationCode.GetLocationMember("判断续接任务").ToValue<bool>())
            {
                webMgrResponse.code = 20000;
                webMgrResponse.message = "不需要判断续接";
                return webMgrResponse;
            }

            if (string.IsNullOrEmpty(data.TaskID))
            {
                data.TaskID = DateTime.Now.ToString("yyyyMMddHHmmssfffffff");
            }

            var taskDataList = Crms.PmsApi.GetTaskDataList();
            if (taskDataList == null)
            {
                webMgrResponse.message = "获取所有任务列表失败";
                return webMgrResponse;
            }

            var stepNullTask = taskDataList.FirstOrDefault(d => d.StepList == null);
            if (stepNullTask != null)
            {
                webMgrResponse.message = $"获取所有任务列表中有步骤为null {stepNullTask.TaskID}";
                return webMgrResponse;
            }

            //查找起点位置是否有任务在最后一步
            var pTask = taskDataList.FirstOrDefault(d =>
            {
                //if (!string.IsNullOrEmpty(d.ParentTaskID)) return false;

                var lasStepData = d.StepList?.OrderBy(s => s.StepID)?.LastOrDefault();
                if (lasStepData == null) return false;

                var lastPos = lasStepData.WareHouseID;
                var lastStepID = lasStepData.StepID;

                if (lastPos != data.LocationCode) return false;
                if (lastStepID != d.CurrentStepID) return false;

                return true;
            });

            if (pTask != null)
            {
                data.ParentTaskId = pTask.TaskID;
                data.Carrier = pTask.Carrier;
                CLog.Instance.TaskLog.WriteInfo($"{data.TaskID} 找到可续接的父任务：{pTask.TaskID} {pTask.Carrier}# {pTask.AllWarehouse.ToShortPosString()}");
            }

            webMgrResponse.code = 20000;
            return webMgrResponse;
        }
        private string GetParentTaskID(PmsTaskInfoDto pmsTaskInfo)
        {
            //查找取货点有没有可以续接的任务
            string startWareHouse = pmsTaskInfo.WareHouseCode;
            string parentTaskID = string.Empty;
            string[] EndWareHouse = ExcelConfig.Instance.TaskRelyList.FirstOrDefault(d => d.StartWareHouse == startWareHouse)?.EndWareHouse;

            if (EndWareHouse == null
                || EndWareHouse.Length == 0)
            {
                return parentTaskID;
            }

            var taskDataList = Crms.PmsApi.GetTaskDataList();
            if (taskDataList == null) return parentTaskID;

            var taskData = taskDataList
                .Where(d =>
                {
                    for (int i = 0; i < EndWareHouse.Length; i++)
                    {
                        string lastPos = d.StepList.OrderBy(f => f.StepID).Last().WareHouseID;
                        string lastPosWhouse = ExcelConfig.Instance.GetRouteConfig(lastPos)?.WareHouseCode;

                        //最后任务点仓库，是开始任务点仓库配置的“可以续接的仓库集合”
                        if (!string.IsNullOrEmpty(lastPosWhouse) && lastPosWhouse.Contains(EndWareHouse[i]))
                        {
                            //满足条件的任务不能是某个任务的父任务
                            if (!taskDataList.Exists(f => f.ParentTaskID == d.TaskID))
                            {
                                d.reserved7 = lastPosWhouse;
                                return true;
                            }
                        }
                    }
                    return false;
                })
                .OrderByDescending(d => d.CurrentStepID)
                .FirstOrDefault();

            if (taskData != null)
            {
                parentTaskID = taskData.TaskID;

                CLog.Instance.TaskLog.WriteInfo($"{Langs.FindParentTask} {parentTaskID} {Langs.OldTaskWarehouse} {taskData.reserved7} -> {Langs.NewTaskWarehouse} {startWareHouse}");
            }

            return parentTaskID;
        }
        protected virtual bool CheckLocaitonStatus(MidTaskInfoEx data, string[] posArray = null)
        {
            if (posArray == null)
            {
                posArray = data.ParametersDic.Values.ToArray().Distinct().ToArray();
            }

            for (int i = 0; i < posArray.Length; i++)
            {
                string location = posArray[i];

                var locationData = locationManager.GetById(location);
                if (locationData == null)
                {
                    continue;
                }

                if (!string.IsNullOrEmpty(locationData.CurrentTaskID))
                {
                    CLog.Instance.TaskLog.WriteInfo($"{data.TaskId} {Langs.TaskOccupancyCheckFailed} {locationData.LocationCode} {locationData.CurrentTaskID}");
                    return false;
                }
            }

            return true;
        }
        protected virtual bool LockLocationStatus(MidTaskInfoEx data, string[] posArray = null)
        {
            if (posArray == null)
            {
                posArray = data.ParametersDic.Values.ToArray().Distinct().ToArray();
            }

            for (int i = 0; i < posArray.Length; i++)
            {
                string location = posArray[i];

                var locationData = locationManager.GetById(location);
                if (locationData == null)
                {
                    continue;
                }

                locationData.CurrentTaskID = data.TaskId;
                bool result = locationManager.UpdateWithTaskId(locationData);
                CLog.Instance.TaskLog.WriteInfo($"{data.TaskId} {location} {Langs.TaskOccupancy}{result.ToChineseString()}");

                if (!result) return result;
            }

            return true;
        }
        protected virtual void RecoverLocationStatus(MidTaskInfoEx data, string[] posArray = null)
        {
            if (posArray == null)
            {
                posArray = data.ParametersDic.Values.ToArray().Distinct().ToArray();
            }

            for (int i = 0; i < posArray.Length; i++)
            {
                string location = posArray[i];

                var locationData = locationManager.GetById(location);
                if (locationData == null)
                {
                    continue;
                }

                if (locationData.Status == LocationStatus.Locked)
                {
                    locationData.Status = LocationStatus.None;
                }

                locationData.CurrentTaskID = string.Empty;
                bool result = locationManager.UpdateWithTaskId(locationData, data.TaskId);
                CLog.Instance.TaskLog.WriteInfo($"{data.TaskId} {location} {Langs.TaskOccupationRelease}{result.ToChineseString()}");

                UnlimitLocation(data.TaskId, location);
            }
        }
        public virtual bool UnlimitLocation(string taskId, string location)
        {
            lock (locker)
            {
                var locationData = locationManager.GetById(location);
                if (locationData == null)
                {
                    return false;
                }

                if (locationData.Status == LocationStatus.Locked
                    || !string.IsNullOrWhiteSpace(locationData.CurrentTaskID))
                {
                    locationData.Status = LocationStatus.None;
                }

                locationData.CurrentTaskID = string.Empty;
                bool result = locationManager.UpdateWithTaskId(locationData, taskId);
                CLog.Instance.TaskLog.WriteInfo($"{taskId} {location} {Langs.TaskOccupationRelease}{result.ToChineseString()}");

                return result;
            }
        }

        public bool LocationHasTask(string location, out string taskId)
        {
            taskId = null;
            var taskDataList = Crms.PmsApi.GetTaskDataList();
            if (taskDataList == null)
            {
                taskId = "获取所有任务列表失败";
                return true;
            }

            var task = taskDataList.FirstOrDefault(d =>
            {
                if (d.TaskState == ETaskState.None)
                {
                    //中间表的任务
                    return true;
                }

                if (d.StepList == null || d.StepList.Count == 0)
                {
                    //没有StepList信息，认为有任务
                    return true;
                }

                var stepData = d.StepList?
                .FirstOrDefault(s => s.WareHouseID == location);

                return stepData != null;
            });

            taskId = task?.TaskID;
            return task != null;
        }
        public bool LocationHasStepTask(string location, out string taskId)
        {
            taskId = null;
            var taskDataList = Crms.PmsApi.GetTaskDataList();
            if (taskDataList == null)
            {
                taskId = Langs.GetTaskDataListFailed;
                return true;
            }

            var task = taskDataList.FirstOrDefault(d =>
            {
                if (d.TaskState == ETaskState.None)
                {
                    //中间表的任务
                    return true;
                }

                if (d.StepList == null || d.StepList.Count == 0)
                {
                    //没有StepList信息，认为有任务
                    return true;
                }

                var stepData = d.StepList?
                .FirstOrDefault(s => s.WareHouseID == location
                && s.SystemStepState <= ESystemStepState.Step_Executing);

                return stepData != null;
            });

            taskId = task?.TaskID;
            return task != null;
        }
        public bool LocationHasStartStepTask(string location, out string taskId)
        {
            taskId = null;
            var taskDataList = Crms.PmsApi.GetTaskDataList();
            if (taskDataList == null)
            {
                taskId = Langs.GetTaskDataListFailed;
                return true;
            }

            var task = taskDataList.FirstOrDefault(d =>
            {
                if (d.TaskState == ETaskState.None)
                {
                    //中间表的任务
                    return true;
                }

                if (d.StepList == null || d.StepList.Count == 0)
                {
                    //没有StepList信息，认为有任务
                    return true;
                }

                var startStepLocation = d.StepList?
                .OrderBy(s => s.StepID)?
                .FirstOrDefault(s => !string.IsNullOrWhiteSpace(s.WareHouseID))?
                .WareHouseID;

                if (startStepLocation != location)
                {
                    //起点不是要判断的任务点，认为没有任务
                    return false;
                }

                var stepData = d.StepList?
                .FirstOrDefault(s => s.WareHouseID == location
                && s.SystemStepState <= ESystemStepState.Step_Executing);

                return stepData != null;
            });

            taskId = task?.TaskID;
            return task != null;
        }
        public virtual string QueryLocationIn(string taskId, string wHouse)
        {
            lock (locker)
            {
                var list = locationManager.GetList(d => d.WareHouseCode == wHouse && d.Status == LocationStatus.Filled && d.CurrentTaskID == taskId);
                if (list.Count == 1)
                {
                    return list[0].LocationCode;
                }

                string nextLocation = locationManager.QueryLocation_In(wHouse);
                if (string.IsNullOrEmpty(nextLocation))
                {
                    return nextLocation;
                }

                var locationData = locationManager.GetById(nextLocation);
                if (locationData == null)
                {
                    return null;
                }

                locationData.CurrentTaskID = taskId;
                var isLocked = locationManager.UpdateWithTaskId(locationData);
                if (!isLocked)
                {
                    return null;
                }

                return nextLocation;
            }
        }
    }
}

