﻿using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Language;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Service
{
    public class PmsTaskServiceSWms : PmsTaskService
    {
        public override WebMgrResponse GetLocationProperty(LocationPropertyQueryDto data)
        {
            WebMgrResponse webMgrResponse = new WebMgrResponse();

            //查询信息
            var locationDataList = Crms.SWmsApi.GetLocationStock(data.WareHouseCode, data.Area, data.LocationCode);
            if (locationDataList == null)
            {
                webMgrResponse.message = Langs.Failed;
                return webMgrResponse;
            }

            locationDataList = locationDataList
                .OrderBy(d => d.Location.warehouseId)
                .ThenBy(d => d.Location.locationId.Length)
                .ThenBy(d => d.Location.locationId);

            var excelConfig = ExcelConfig.Instance;
            var dtoList = new List<LocationPropertyDto>();
            foreach (var item in locationDataList)
            {
                LocationPropertyDto dto = new LocationPropertyDto();
                dto.WareHouseCode = item.Location.warehouseId;
                dto.LocationCode = item.Location.locationId;
                dto.Mark = excelConfig.GetRouteConfig(dto.LocationCode)?.StationID ?? 0;
                dto.Area = item.Location.areaId;
                //Enum.TryParse(item.Location.locationStatus, out LocationStatus locationStatus);
                //dto.Status = locationStatus;

                dto.Status = item.Location.locationStatus == "0" ? LocationStatus.None : item.Stock == null ? LocationStatus.Empty : LocationStatus.Filled;
                dto.MaterialId = item.Stock?.goodsCode;
                dto.MaterialNumber = item.Stock?.goodsNumber.ToString();
                dto.CurrentTaskID = item.Location.lockMark;

                dto.TargetWareHouseCode = dto.LocationCode.GetLocationMember("目标仓库");
                dto.SourceWareHouseCode = dto.LocationCode.GetLocationMember("源仓库");

                dtoList.Add(dto);
            }

            webMgrResponse.code = 20000;
            //webMgrResponse.data = Newtonsoft.Json.JsonConvert.SerializeObject(dtoList);
            webMgrResponse.data = dtoList;
            webMgrResponse.message = Langs.Successful;
            return webMgrResponse;
        }
        public override WebMgrResponse UpdateLocationProperty(LocationPropertyUpdateDto data)
        {
            lock (locker)
            {
                WebMgrResponse webMgrResponse = new WebMgrResponse();

                //查询信息
                var locationData = Crms.SWmsApi.GetLocationStockByLocationId(data.LocationCode);
                if (locationData == null)
                {
                    webMgrResponse.message = $"{data.LocationCode} {Langs.LocationGetInfoFailed}";
                    return webMgrResponse;
                }

                //有任务的位置不能修改
                if (!string.IsNullOrEmpty(locationData.Location.lockMark))
                {
                    if (LocationHasStepTask(data.LocationCode, out string taskId))
                    {
                        webMgrResponse.message = $"{data.LocationCode} {Langs.LocationOccupiedByTask} {taskId}";
                        return webMgrResponse;
                    }
                }

                bool result = false;
                string content = null;

                //处理未知状态
                var statusCode = data.Status == LocationStatus.None ? data.Status.ToValue<int>().ToString() : string.Empty;
                if (locationData.Location.locationStatus != statusCode)
                {
                    locationData.Location.locationStatus = statusCode;
                    result = Crms.SWmsApi.UpdateLocation(locationData.Location, out content);
                    if (!result)
                    {
                        webMgrResponse.message = $"{data.LocationCode} 更新库位状态失败 {(result ? null : content)}";
                        return webMgrResponse;
                    }
                }

                //解锁库位
                result = Crms.SWmsApi.UpdateLocationLockMark(data.LocationCode, locationData.Location.lockMark, null, out content);
                if (!result)
                {
                    webMgrResponse.message = $"{data.LocationCode} 解锁库位失败 {(result ? null : content)}";
                    return webMgrResponse;
                }

                //根据状态设置货物
                if (data.Status == LocationStatus.Filled && string.IsNullOrWhiteSpace(data.MaterialId))
                {
                    data.MaterialId = "默认货物";
                }
                if (data.Status == LocationStatus.Empty && !string.IsNullOrWhiteSpace(data.MaterialId))
                {
                    data.MaterialId = null;
                }

                //处理库存
                if (string.IsNullOrWhiteSpace(data.MaterialId))
                {
                    //没有库存直接返回
                    if (locationData.Stock == null)
                    {
                        webMgrResponse.code = 20000;
                        webMgrResponse.message = $"{data.LocationCode} 库位货物已清空";
                        return webMgrResponse;
                    }

                    //清空库位货物
                    result = Crms.SWmsApi.StockOutManualByLocationId(data.LocationCode, out content);

                    webMgrResponse.code = result ? 20000 : 9999;
                    webMgrResponse.message = $"{data.LocationCode} 清空库位货物 {result.ToChineseString()} {(result ? null : content)}";

                    CLog.Instance.SystemLog.WirteInfoCaller(webMgrResponse.message);
                    return webMgrResponse;
                }
                else
                {
                    //添加货物
                    var goodsList = Crms.SWmsApi.GetGoodsByCondition(new SWmsGoodsByCondition() { goodsCode = data.MaterialId }, out content);
                    if (goodsList == null)
                    {
                        webMgrResponse.message = $"获取货物失败 {data.MaterialId} {content}";
                        return webMgrResponse;
                    }

                    if (!goodsList.Exists(d => d.goodsCode == data.MaterialId))
                    {
                        var freightShapeList = Crms.SWmsApi.GetAllFreightShape();
                        if (freightShapeList == null)
                        {
                            webMgrResponse.message = "获取所有模型失败";
                            return webMgrResponse;
                        }

                        if (freightShapeList.Count == 0)
                        {
                            webMgrResponse.message = "找不到任何货物模型";
                            return webMgrResponse;
                        }

                        var data2 = new SWmsGoods();
                        data2.goodsCode = data.MaterialId;
                        data2.goodsName = data.MaterialId;
                        data2.goodsShape = freightShapeList.FirstOrDefault()?.id ?? 0;

                        result = Crms.SWmsApi.AddGoods(data2, out content);
                        if (!result)
                        {
                            webMgrResponse.message = $"添加货物失败 {content}";
                            return webMgrResponse;
                        }
                    }

                    //货物数量转换
                    var materialNumber = 1;
                    if (!string.IsNullOrWhiteSpace(data.MaterialNumber) && !int.TryParse(data.MaterialNumber, out materialNumber))
                    {
                        webMgrResponse.message = $"货物数量转换失败 {data.MaterialNumber}";
                        return webMgrResponse;
                    }

                    //设置库存
                    if (locationData.Stock == null)
                    {
                        var stock = new SWmsStock();
                        stock.supporterType = 0;
                        stock.supporterId = data.LocationCode;
                        stock.goodsCode = data.MaterialId;
                        stock.goodsNumber = materialNumber;

                        result = Crms.SWmsApi.StockInManual(stock, out content);

                        webMgrResponse.code = result ? 20000 : 9999;
                        webMgrResponse.message = $"{data.LocationCode} 设置库位货物 {data.MaterialId} {materialNumber} {result.ToChineseString()} {(result ? null : content)}";

                        CLog.Instance.SystemLog.WirteInfoCaller(webMgrResponse.message);
                        return webMgrResponse;
                    }
                    else
                    {
                        //清除多余库存
                        var unnecessaryStocks = locationData.StockList.Where(d => d.id != locationData.Stock.id);
                        foreach (var item in unnecessaryStocks)
                        {
                            result = Crms.SWmsApi.StockOutManualById(item.id, out content);
                            if (!result)
                            {
                                webMgrResponse.code = result ? 20000 : 9999;
                                webMgrResponse.message = $"{data.LocationCode} 清除库位多余货物 {item.id} {result.ToChineseString()} {(result ? null : content)}";
                                return webMgrResponse;
                            }
                        }

                        //库存一致时直接返回
                        if (locationData.Stock.goodsCode == data.MaterialId
                            && locationData.Stock.goodsNumber == materialNumber)
                        {
                            webMgrResponse.code = 20000;
                            webMgrResponse.message = $"{data.LocationCode} 库位货物已设置 {data.MaterialId} {materialNumber}";
                            return webMgrResponse;
                        }

                        //更新库存
                        locationData.Stock.goodsCode = data.MaterialId;
                        locationData.Stock.goodsNumber = materialNumber;
                        result = Crms.SWmsApi.UpdateStockManual(locationData.Stock, out content);

                        webMgrResponse.code = result ? 20000 : 9999;
                        webMgrResponse.message = $"{data.LocationCode} 更新库位货物 {data.MaterialId} {materialNumber} {result.ToChineseString()} {(result ? null : content)}";

                        CLog.Instance.SystemLog.WirteInfoCaller(webMgrResponse.message);
                        return webMgrResponse;
                    }
                }
            }
        }

        protected override WebMgrResponse PmsTaskInfoCheck(PmsTaskInfoDto pmsTaskInfo)
        {
            var webMgrResponse = new WebMgrResponse();

            //匹配取货点
            if (string.IsNullOrEmpty(pmsTaskInfo.LocationCode))
            {
                if (string.IsNullOrEmpty(pmsTaskInfo.WareHouseCode))
                {
                    webMgrResponse.message = Langs.SourceWarehouseAndSourceLocationAllEmpty;
                    return webMgrResponse;
                }

                string location = Crms.SWmsApi.GetLocationIdOut(pmsTaskInfo.WareHouseCode, pmsTaskInfo.MaterialId);
                if (string.IsNullOrEmpty(location))
                {
                    webMgrResponse.message = $"{Langs.SourceLocationMatchFailed} {Langs.TipWareHouseCode}{pmsTaskInfo.WareHouseCode} {Langs.TipMaterialId}{pmsTaskInfo.MaterialId}";
                    return webMgrResponse;
                }
                pmsTaskInfo.LocationCode = location;
            }

            //查询取货点信息
            var uLocationData = Crms.SWmsApi.GetLocationStockByLocationId(pmsTaskInfo.LocationCode);
            if (uLocationData == null)
            {
                webMgrResponse.message = $"{Langs.SourceLocationGetInfoFailed} {pmsTaskInfo.LocationCode}";
                return webMgrResponse;
            }
            if (!uLocationData.Location.enable)
            {
                webMgrResponse.message = $"取货仓位已禁用 {uLocationData.Location.locationId}";
                return webMgrResponse;
            }
            if (uLocationData.Stock == null || uLocationData.Location.locationStatus == "0")
            {
                webMgrResponse.message = $"{Langs.SourceLocationMustBeFilled} {uLocationData.Location.locationId}";
                return webMgrResponse;
            }
            if (!string.IsNullOrEmpty(uLocationData.Location.lockMark))
            {
                webMgrResponse.message = $"{Langs.SourceLocationOccupiedByTask} {uLocationData.Location.locationId} {uLocationData.Location.lockMark}";
                return webMgrResponse;
            }
            if (!Crms.SWmsApi.CheckLocation(uLocationData, out var blockLocation1))
            {
                webMgrResponse.message = $"{Langs.SourceLocationObstacleDetectionFailed} {uLocationData.Location.locationId} => {blockLocation1?.Location.locationId}";
                return webMgrResponse;
            }
            if (!Crms.SWmsApi.CheckLocationColumnNoTask(uLocationData, out var taskLocation1))
            {
                webMgrResponse.message = $"{Langs.SourceLocationHaveColumnTask} {uLocationData.Location.locationId} （{taskLocation1?.Location.locationId}{Langs.TaskID}{taskLocation1?.Location.lockMark}）";
                return webMgrResponse;
            }


            //匹配放货点
            if (string.IsNullOrEmpty(pmsTaskInfo.TargetLocationCode))
            {
                if (string.IsNullOrEmpty(pmsTaskInfo.TargetWareHouseCode))
                {
                    webMgrResponse.message = Langs.TargetWarehouseAndTargetLocationAllEmpty;
                    return webMgrResponse;
                }


                string location = Crms.SWmsApi.GetLocationIdIn(pmsTaskInfo.TargetWareHouseCode);
                if (string.IsNullOrEmpty(location))
                {
                    webMgrResponse.message = $"{Langs.TargetLocationMatchFailed} {Langs.TipWareHouseCode}{pmsTaskInfo.TargetWareHouseCode}";
                    return webMgrResponse;
                }
                pmsTaskInfo.TargetLocationCode = location;
            }

            //查询放货点信息
            var dLocationData = Crms.SWmsApi.GetLocationStockByLocationId(pmsTaskInfo.TargetLocationCode);
            if (dLocationData == null)
            {
                webMgrResponse.message = $"{Langs.TargetLocationGetInfoFailed} {pmsTaskInfo.TargetLocationCode}";
                return webMgrResponse;
            }
            if (!dLocationData.Location.enable)
            {
                webMgrResponse.message = $"放货仓位已禁用 {uLocationData.Location.locationId}";
                return webMgrResponse;
            }
            if (dLocationData.Stock != null || dLocationData.Location.locationStatus == "0")
            {
                webMgrResponse.message = $"{Langs.TargetLocationMustBeEmpty} {dLocationData.Location.locationId}";
                return webMgrResponse;
            }
            if (!string.IsNullOrEmpty(uLocationData.Location.lockMark))
            {
                webMgrResponse.message = $"{Langs.TargetLocationOccupiedByTask} {dLocationData.Location.locationId} {dLocationData.Location.lockMark}";
                return webMgrResponse;
            }
            if (!Crms.SWmsApi.CheckLocation(dLocationData, out var blockLocation2))
            {
                webMgrResponse.message = $"{Langs.TargetLocationObstacleDetectionFailed} {dLocationData.Location.locationId} => {blockLocation2?.Location.locationId}";
                return webMgrResponse;
            }
            if (!Crms.SWmsApi.CheckLocationColumnNoTask(dLocationData, out var taskLocation2))
            {
                webMgrResponse.message = $"{Langs.TargetLocationHaveColumnTask} {dLocationData.Location.locationId} （{taskLocation2?.Location.locationId}{Langs.TaskID}{taskLocation2?.Location.lockMark}）";
                return webMgrResponse;
            }



            pmsTaskInfo.WareHouseCode = uLocationData.Location.warehouseId;
            pmsTaskInfo.TargetWareHouseCode = dLocationData.Location.warehouseId;

            webMgrResponse.code = 20000;
            webMgrResponse.data = pmsTaskInfo;
            return webMgrResponse;
        }

        protected override bool CheckLocaitonStatus(MidTaskInfoEx data, string[] posArray = null)
        {
            if (posArray == null)
            {
                posArray = data.ParametersDic.Values.ToArray().Distinct().ToArray();
            }

            for (int i = 0; i < posArray.Length; i++)
            {
                string location = posArray[i];

                var locationData = Crms.SWmsApi.GetLocationStockByLocationId(location);
                if (locationData == null)
                {
                    continue;
                }

                if (!string.IsNullOrEmpty(locationData.Location.lockMark))
                {
                    CLog.Instance.TaskLog.WriteInfo($"{data.TaskId} {Langs.TaskOccupancyCheckFailed} {locationData.Location.locationId} {locationData.Location.locationId}");
                    return false;
                }
            }

            return true;
        }
        protected override bool LockLocationStatus(MidTaskInfoEx data, string[] posArray = null)
        {
            if (posArray == null)
            {
                posArray = data.ParametersDic.Values.ToArray().Distinct().ToArray();
            }

            for (int i = 0; i < posArray.Length; i++)
            {
                string location = posArray[i];

                var locationData = Crms.SWmsApi.GetLocationStockByLocationId(location);
                if (locationData == null)
                {
                    continue;
                }

                bool result = Crms.SWmsApi.UpdateLocationLockMark(location, null, data.TaskId, out var content);
                CLog.Instance.TaskLog.WriteInfo($"{data.TaskId} {location} {Langs.TaskOccupancy}{result.ToChineseString()} {(result ? null : content)}");

                if (!result) return result;
            }

            return true;
        }
        protected override void RecoverLocationStatus(MidTaskInfoEx data, string[] posArray = null)
        {
            if (posArray == null)
            {
                posArray = data.ParametersDic.Values.ToArray().Distinct().ToArray();
            }

            for (int i = 0; i < posArray.Length; i++)
            {
                string location = posArray[i];

                var locationData = Crms.SWmsApi.GetLocationStockByLocationId(location);
                if (locationData == null)
                {
                    continue;
                }

                bool result = Crms.SWmsApi.UpdateLocationLockMark(location, data.TaskId, null, out var content);
                CLog.Instance.TaskLog.WriteInfo($"{data.TaskId} {location} {Langs.TaskOccupationRelease}{result.ToChineseString()} {(result ? null : content)}");
            }
        }
        public override bool UnlimitLocation(string taskId, string location)
        {
            lock (locker)
            {
                var locationData = Crms.SWmsApi.GetLocationStockByLocationId(location);
                if (locationData == null)
                {
                    CLog.Instance.TaskLog.WriteInfo($"{taskId} {location} 获取库位信息失败");
                    return false;
                }
                if (string.IsNullOrWhiteSpace(locationData.Location.lockMark))
                {
                    CLog.Instance.TaskLog.WriteInfo($"{taskId} {location} 任务占用无需解除");
                    return false;
                }
                if (locationData.Location.lockMark != taskId)
                {
                    CLog.Instance.TaskLog.WriteInfo($"{taskId} {location} 任务占用不匹配 {locationData.Location.lockMark}");
                    return false;
                }

                //locationData.Location.locationStatus = LocationStatus.None.ToValue<int>().ToString();
                //var result2 = Crms.SWmsApi.UpdateLocation(locationData.Location, out var content2);
                //if (!result2)
                //{
                //    CLog.Instance.TaskLog.WriteInfo($"{taskId} {location} 更新库位状态失败 {(result2 ? null : content2)}");
                //}

                var result = Crms.SWmsApi.UpdateLocationLockMark(location, taskId, null, out var content);
                CLog.Instance.TaskLog.WriteInfo($"{taskId} {location} {Langs.TaskOccupationRelease}{result.ToChineseString()} {(result ? null : content)}");

                return result;
            }
        }
        public override string QueryLocationIn(string taskId, string wHouse)
        {
            lock (locker)
            {
                var list = Crms.SWmsApi.GetLocationStockByWarehouseId(wHouse);
                if (list == null) return null;

                var data = list.FirstOrDefault(d => d.Location.lockMark == taskId);
                if (data != null)
                {
                    return data.Location.locationId;
                }

                var nextLocationData = Crms.SWmsApi.GetLocationStockIn(wHouse);
                if (nextLocationData == null)
                {
                    return null;
                }

                var nextLocation = nextLocationData.Location.locationId;
                bool result = Crms.SWmsApi.UpdateLocationLockMark(nextLocation, null, taskId, out var content);
                if (!result)
                {
                    return null;
                }

                return nextLocation;
            }
        }
    }
}
