﻿using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models.Crms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Service
{
    public class SWmsApi
    {
        private RestApi restApi = new RestApi();

        public bool Get<T>(string routes, object pars, object body, out T data)
        {
            data = default(T);
            try
            {
                var url = Common.Function.AppSetting.GetCrmsApiUrl(routes);
                var res = restApi.Get<WebMgrResponse<T>>(url, pars, body, out string content);

                data = res.data;
                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }
            return false;
        }
        public bool Post<T>(string routes, object pars, object body, out T data, out string content)
        {
            data = default(T);
            content = null;
            try
            {
                var url = Common.Function.AppSetting.GetCrmsApiUrl(routes);
                var res = restApi.Post<WebMgrResponse<T>>(url, pars, body, out content);
                //CLog.Instance.SystemLog.WriteDebug($"Post {url}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");

                data = res.data;
                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }

            return false;
        }
        public bool Put<T>(string routes, object pars, object body, out T data, out string content)
        {
            data = default(T);
            content = null;
            try
            {
                var url = Common.Function.AppSetting.GetCrmsApiUrl(routes);
                var res = restApi.Put<WebMgrResponse<T>>(url, pars, body, out content);
                //CLog.Instance.SystemLog.WriteDebug($"Put {url}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");

                data = res.data;
                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }

            return false;
        }


        public List<SWmsWarehouse> GetAllWarehouse()
        {
            if (!Get("wms/warehouse-config/getAllWarehouse", null, null, out List<SWmsWarehouse> dataList)) return null;
            return dataList;
        }
        public List<SWmsArea> GetAllArea()
        {
            if (!Get("wms/area-config/getAllArea", null, null, out List<SWmsArea> dataList)) return null;
            return dataList;
        }
        public List<SWmsLocationType> GetAllLocationType()
        {
            if (!Get("wms/location-type-config/getAllLocationType", null, null, out List<SWmsLocationType> dataList)) return null;
            return dataList;
        }
        public List<SWmsLocationStatus> GetAllLocationStatus()
        {
            if (!Get("wms/location-status-config/getAllLocationStatus", null, null, out List<SWmsLocationStatus> dataList)) return null;
            return dataList;
        }
        public List<SWmsFreightShape> GetAllFreightShape()
        {
            if (!Get("wms/freight-shape-config/getAllFreightShape", null, null, out List<SWmsFreightShape> dataList)) return null;
            return dataList;
        }
        public List<SWmsGoodsType> GetAllGoodsType()
        {
            if (!Get("wms/goods-type-config/getAllGoodsType", null, null, out List<SWmsGoodsType> dataList)) return null;
            return dataList;
        }
        public List<SWmsAttribute> GetAllAttribute()
        {
            if (!Get("wms/attribute-config/getAllAttribute", null, null, out List<SWmsAttribute> dataList)) return null;
            return dataList;
        }


        public bool AddWarehouse(SWmsWarehouse data, out string content)
        {
            return Post("warehouse-config/addWarehouse", null, data, out object _, out content);
        }
        public bool UpdateWarehouse(SWmsWarehouse data, out string content)
        {
            return Post("warehouse-config/updateWarehouse", null, data, out object _, out content);
        }
        public bool AddArea(SWmsArea data, out string content)
        {
            return Post("area-config/addArea", null, data, out object _, out content);
        }
        public bool UpdateArea(SWmsArea data, out string content)
        {
            return Post("area-config/updateArea", null, data, out object _, out content);
        }
        public bool AddLocationType(SWmsLocationType data, out string content)
        {
            return Post("location-type/addLocationType", null, data, out object _, out content);
        }
        public bool UpdateLocationType(SWmsLocationType data, out string content)
        {
            return Post("location-type/updateLocationType", null, data, out object _, out content);
        }
        public bool AddLocationStatus(SWmsLocationStatus data, out string content)
        {
            return Post("location-status/addLocationStatus", null, data, out object _, out content);
        }
        public bool UpdateLocationStatus(SWmsLocationStatus data, out string content)
        {
            return Post("location-status/updateLocationStatus", null, data, out object _, out content);
        }
        public bool AddFreightShape(SWmsFreightShape data, out string content)
        {
            return Post("freight-shape/", null, data, out object _, out content);
        }
        public bool UpdateFreightShape(SWmsFreightShape data, out string content)
        {
            return Put("freight-shape/", null, data, out object _, out content);
        }
        public bool AddGoodsType(SWmsGoodsType data, out string content)
        {
            return Post("goods-type/addGoodsType", null, data, out object _, out content);
        }
        public bool UpdateGoodsType(SWmsGoodsType data, out string content)
        {
            return Post("goods-type/updateGoodsType", null, data, out object _, out content);
        }
        public bool AddAttribute(SWmsAttribute data, out string content)
        {
            return Post("attribute-config/addAttribute", null, data, out object _, out content);
        }
        public bool UpdateAttribute(SWmsAttribute data, out string content)
        {
            return Post("attribute-config/updateAttribute", null, data, out object _, out content);
        }


        public bool AddRack(SWmsRack data, out string content)
        {
            return Post("wms/rack/addRack", null, data, out object _, out content);
        }
        public bool UpdateRack(SWmsRack data, out string content)
        {
            return Post("wms/rack/updateRack", null, data, out object _, out content);
        }
        public bool DeleteRack(string rackCode, out string content)
        {
            return Post("wms/rack/deleteRack", null, new { rackCode }, out object _, out content);
        }
        public List<SWmsRack> GetAllRack()
        {
            if (!Get("wms/rack/getAllRack", null, null, out List<SWmsRack> dataList)) return null;
            return dataList;
        }
        public List<SWmsRack> GetEmptyRack()
        {
            if (!Get("wms/rack/getEmptyRack", null, null, out List<SWmsRack> dataList)) return null;
            return dataList;
        }


        public bool AddGoods(SWmsGoods data, out string content)
        {
            return Post("wms/goods/addGoods", null, data, out object _, out content);
        }
        public bool UpdateGoods(SWmsGoods data, out string content)
        {
            return Post("wms/goods/updateGoods", null, data, out object _, out content);
        }
        public bool DeleteGoods(string goodsCode, out string content)
        {
            return Post("wms/goods/deleteGoods", null, new { goodsCode }, out object _, out content);
        }
        public List<SWmsGoods> GetAllGoods()
        {
            if (!Get("wms/goods/getAllGoods", null, null, out List<SWmsGoods> dataList)) return null;
            return dataList;
        }
        public List<SWmsGoods> GetGoodsByCondition(SWmsGoodsByCondition data, out string content)
        {
            if (!Post("wms/goods/getGoodsByCondition", null, data, out List<SWmsGoods> dataList, out content)) return null;
            return dataList;
        }


        public bool AddLocation(SWmsLocation data, out string content)
        {
            return Post("wms/location/addLocation", null, data, out object _, out content);
        }
        public bool UpdateLocation(SWmsLocation data, out string content)
        {
            return Post("wms/location/updateLocation", null, data, out object _, out content);
        }
        public bool DeleteLocation(string locationId, out string content)
        {
            return Post("wms/location/deleteLocation", null, new { locationId }, out object _, out content);
        }
        public List<SWmsLocation> GetAllLocation()
        {
            if (!Get("wms/location/getAllLocation", null, null, out List<SWmsLocation> dataList)) return null;
            return dataList;
        }
        public List<SWmsLocation> GetLocationByCondition(SWmsLocationByCondition data, out string content)
        {
            if (!Post("wms/location/getLocationByCondition", null, data, out List<SWmsLocation> dataList, out content)) return null;
            return dataList;
        }
        public List<SWmsWarehouseGroup> GetAllWarehouseGroup()
        {
            if (!Get("wms/location/getAllWarehouseGroup", null, null, out List<SWmsWarehouseGroup> dataList)) return null;
            return dataList;
        }
        public bool UpdateLocationLockMark(SWmsLocationLockMark data, out string content)
        {
            return Post("wms/location/updateLocationLockMark", null, data, out object _, out content);
        }
        public List<SWmsLocation> GetEmptyLocation()
        {
            if (!Get("wms/location/getEmptyLocation", null, null, out List<SWmsLocation> dataList)) return null;
            return dataList;
        }


        public bool StockIn(SWmsStock data, out string content)
        {
            return Post("wms/stock/stockIn", null, data, out object _, out content);
        }
        public bool StockOut(SWmsStockOut data, out string content)
        {
            return Post("wms/stock/stockOut", null, data, out object _, out content);
        }
        public bool StockMove(SWmsStockMove data, out string content)
        {
            return Post("wms/stock/stockMove", null, data, out object _, out content);
        }
        public bool UpdateStock(SWmsStock data, out string content)
        {
            return Post("wms/stock/updateStock", null, data, out object _, out content);
        }
        public bool StockOutById(SWmsStockOutById data, out string content)
        {
            return Post("wms/stock/stockOutById", null, data, out object _, out content);
        }
        public List<SWmsStock> GetStockByCondition(SWmsStockByCondition data, out string content)
        {
            if (!Post("wms/stock/getStockByCondition", null, data, out List<SWmsStock> dataList, out content)) return null;
            return dataList;
        }
        public SWmsStockRecordPage StockRecordFindPage(SWmsStockRecordCondition data, out string content)
        {
            if (!Post("wms/stock-record/findPage", null, data, out SWmsStockRecordPage dataList, out content)) return null;
            return dataList;
        }



        //function
        public SWmsStock GetStockByAgvId(int agvId, out string content)
        {
            return Crms.SWmsApi.GetStockByCondition(new SWmsStockByCondition() { carrierId = agvId.ToString() }, out content)?.FirstOrDefault();
        }
        public bool UpdateLocationLockMark(string locationId, string beforeLockMark, string afterLockMark, out string content)
        {
            return UpdateLocationLockMark(new SWmsLocationLockMark() { locationId = locationId, beforeLockMark = beforeLockMark, afterLockMark = afterLockMark }, out content);
        }
        public bool UpdateStockManual(SWmsStock data, out string content)
        {
            data.recordType = 6;
            data.operationSource = "pms";

            return UpdateStock(data, out content);
        }
        public bool StockInManual(SWmsStock data, out string content)
        {
            data.recordType = 0;
            data.operationSource = "pms";

            return StockIn(data, out content);
        }
        public bool StockInAuto(SWmsStock data, out string content)
        {
            data.recordType = 3;
            data.operationSource = "pms";

            return StockIn(data, out content);
        }
        public bool StockOutManualByLocationId(string locationId, out string content)
        {
            return StockOut(new SWmsStockOut() { supporterType = 0, supporterId = locationId, recordType = 1, operationSource = "pms" }, out content);
        }
        public bool StockOutManualById(string id, out string content)
        {
            return StockOutById(new SWmsStockOutById() { id = id, recordType = 1, operationSource = "pms" }, out content);
        }
        public bool StockOutAutoByLocationId(string locationId, string taskId, out string content)
        {
            return StockOut(new SWmsStockOut() { supporterType = 0, supporterId = locationId, recordType = 4, operationSource = "pms", taskId = taskId }, out content);
        }
        public bool StockOutAutoByAgvId(int agvId, string taskId, out string content)
        {
            return StockOut(new SWmsStockOut() { supporterType = 1, supporterId = agvId.ToString(), recordType = 4, operationSource = "pms", taskId = taskId }, out content);
        }
        public bool StockMoveAutoLocationToAgv(string locationId, int agvId, string taskId, out string content)
        {
            var data = new SWmsStockMove()
            {
                currentSupporterType = 0,
                currentSupporterId = locationId,
                targetSupporterType = 1,
                targetSupporterId = agvId.ToString(),
                recordType = 5,
                operationSource = "pms",
                taskId = taskId
            };

            return StockMove(data, out content);
        }
        public bool StockMoveAutoAgvToLocation(int agvId, string locationId, string taskId, out string content)
        {
            var data = new SWmsStockMove()
            {
                currentSupporterType = 1,
                currentSupporterId = agvId.ToString(),
                targetSupporterType = 0,
                targetSupporterId = locationId,
                recordType = 5,
                operationSource = "pms",
                taskId = taskId
            };

            return StockMove(data, out content);
        }

        public IEnumerable<SWmsLocationStock> GetLocationStock(SWmsLocationByCondition locationByCondition, SWmsStockByCondition stockByCondition)
        {
            var locationList = Crms.SWmsApi.GetLocationByCondition(locationByCondition, out var content);
            if (locationList == null || locationList.Count == 0) return null;

            var goodsList = Crms.SWmsApi.GetStockByCondition(stockByCondition, out content);
            if (goodsList == null) return null;

            return locationList.GroupJoin(goodsList, o => o.locationId, i => i.supporterId, (o, i) => new SWmsLocationStock() { Location = o, Stock = i.FirstOrDefault(), StockList = i });
        }
        public IEnumerable<SWmsLocationStock> GetLocationStock(string warehouseId, string areaId, string locationId)
        {
            var locationByCondition = new SWmsLocationByCondition() { warehouseId = warehouseId, areaId = areaId, locationId = locationId };
            var stockByCondition = new SWmsStockByCondition() { warehouseId = warehouseId, areaId = areaId, locationId = locationId };
            return GetLocationStock(locationByCondition, stockByCondition);
        }
        public IEnumerable<SWmsLocationStock> GetLocationStockByWarehouseId(string warehouseId)
        {
            var locationByCondition = new SWmsLocationByCondition() { warehouseId = warehouseId };
            var stockByCondition = new SWmsStockByCondition() { warehouseId = warehouseId };
            return GetLocationStock(locationByCondition, stockByCondition);
        }
        public SWmsLocationStock GetLocationStockByLocationId(string locationId)
        {
            var locationByCondition = new SWmsLocationByCondition() { locationId = locationId };
            var stockByCondition = new SWmsStockByCondition() { locationId = locationId };
            return GetLocationStock(locationByCondition, stockByCondition)?.FirstOrDefault();
        }
        public SWmsLocationStock GetLocationStockOut(string warehouseId, string material = null)
        {
            var wLocationDatas = GetLocationStockByWarehouseId(warehouseId);
            if (wLocationDatas == null) return null;

            var locationData = wLocationDatas
                .Where(d => d.Location.enable && string.IsNullOrWhiteSpace(d.Location.lockMark) && d.Stock != null && d.Location.locationStatus != "0")
                .Where(d => material == null ? true : d.Stock.goodsCode == material)
                .OrderBy(d => d.Location.column)
                .ThenByDescending(d => d.Location.row)
                .ThenBy(d => d.Stock.updateTime)
                .FirstOrDefault(d => CheckLocation(wLocationDatas, d, out var _) && CheckLocationColumnNoTask(wLocationDatas, d, out var _));

            return locationData;
        }
        public SWmsLocationStock GetLocationStockIn(string warehouseId)
        {
            var wLocationDatas = GetLocationStockByWarehouseId(warehouseId);
            if (wLocationDatas == null) return null;

            var locationData = wLocationDatas
                .Where(d => d.Location.enable && string.IsNullOrWhiteSpace(d.Location.lockMark) && d.Stock == null && d.Location.locationStatus != "0")
                .OrderBy(d => d.Location.column)
                .ThenByDescending(d => d.Location.row)
                .FirstOrDefault(d => CheckLocation(wLocationDatas, d, out var _) && CheckLocationColumnNoTask(wLocationDatas, d, out var _));

            return locationData;
        }
        public string GetLocationIdOut(string warehouseId, string material = null)
        {
            return GetLocationStockOut(warehouseId, material)?.Location?.locationId;
        }
        public string GetLocationIdIn(string warehouseId)
        {
            return GetLocationStockIn(warehouseId)?.Location?.locationId;
        }
        public bool CheckLocation(SWmsLocationStock data, out SWmsLocationStock blockLocation)
        {
            blockLocation = null;
            var wLocationDatas = GetLocationStockByWarehouseId(data.Location.warehouseId);
            if (wLocationDatas == null) return false;

            return CheckLocation(wLocationDatas, data, out blockLocation);
        }
        public bool CheckLocationColumnNoTask(SWmsLocationStock data, out SWmsLocationStock taskLocation)
        {
            taskLocation = null;
            var wLocationDatas = GetLocationStockByWarehouseId(data.Location.warehouseId);
            if (wLocationDatas == null) return false;

            return CheckLocationColumnNoTask(wLocationDatas, data, out taskLocation);
        }
        public bool CheckLocation(IEnumerable<SWmsLocationStock> wLocationDatas, SWmsLocationStock data, out SWmsLocationStock blockLocation)
        {
            blockLocation = null;
            if (data == null) return false;
            if (string.IsNullOrWhiteSpace(data.Location.column) && string.IsNullOrWhiteSpace(data.Location.row)) return true;

            //检查出库的线路上是否都是空状态（障碍检测）
            blockLocation = wLocationDatas
                .FirstOrDefault(d => d.Location.column == data.Location.column
                && d.Location.row.CompareTo(data.Location.row) < 0
                && (d.Stock != null || !d.Location.enable || d.Location.locationStatus == "0"));

            return blockLocation == null;
        }
        public bool CheckLocationColumnNoTask(IEnumerable<SWmsLocationStock> wLocationDatas, SWmsLocationStock data, out SWmsLocationStock taskLocation)
        {
            taskLocation = null;
            if (data == null) return false;
            if (string.IsNullOrWhiteSpace(data.Location.column) && string.IsNullOrWhiteSpace(data.Location.row)) return true;

            //检查库位相同列上是否全部没有任务（同列任务检测）
            taskLocation = wLocationDatas
                .FirstOrDefault(d => d.Location.column == data.Location.column
                && d.Location.row != data.Location.row
                && !string.IsNullOrWhiteSpace(d.Location.lockMark));

            return taskLocation == null;
        }
    }
}
