﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskBookEvent
{

    public class AsyncBehaviorPauseCarrier : AsyncBehavior
    {
        public string EStopAddr { get; set; } = "PLC急停";
        public string InterLockAddr { get; set; }

        public override bool CustomOperator(TaskData taskData, StepData stepInfo)
        {
            var routeConfig = ExcelConfig.Instance.GetRouteConfig(stepInfo.WareHouseID);
            if (routeConfig == null)
            {
                Log(taskData, stepInfo, "查找库位配置信息失败");
                return false;
            }
            if (string.IsNullOrEmpty(routeConfig.PlcIpAddr))
            {
                Log(taskData, stepInfo, "PlcIp地址为空");
                return false;
            }

            var plc = routeConfig.PlcReadWrite;

            string eStopAddr = routeConfig.LocationCode.GetLocationMember(EStopAddr);
            string interLockAddr = routeConfig.LocationCode.GetLocationMember(InterLockAddr);

            var tempResult = plc.ReadBool(eStopAddr);
            bool eStop = tempResult.IsSuccess && tempResult.Content;
            
            var tempResult2 = plc.ReadBool(interLockAddr);
            bool interLock = tempResult2.IsSuccess && tempResult2.Content;

            //无信号则暂停agv
            if (!eStop || !interLock)
            {
                bool result = Crms.PmsApi.CarrierPause(taskData.Carrier);
                Log(taskData, stepInfo, $"暂停小车{result.ToChineseString()} {EStopAddr}{eStopAddr}={eStop} {InterLockAddr}{interLockAddr}={interLock}");
            }

            if (eStop && interLock)
            {
                bool result = Crms.PmsApi.CarrierResume(taskData.Carrier);
                Log(taskData, stepInfo, $"恢复小车{result.ToChineseString()} {EStopAddr}{eStopAddr}={eStop} {InterLockAddr}{interLockAddr}={interLock}");
            }

            return false;
        }
    }

    public class AsyncBehaviorPauseCarrier正在进入 : AsyncBehaviorPauseCarrier
    {
        public AsyncBehaviorPauseCarrier正在进入()
        {
            InterLockAddr = "PLC进入互锁";
        }
    }

    public class AsyncBehaviorPauseCarrier正在离开 : AsyncBehaviorPauseCarrier
    {
        public AsyncBehaviorPauseCarrier正在离开()
        {
            InterLockAddr = "PLC离开互锁";
        }
    }
}
