﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskBookEvent
{
    public class PlcBehaviorOperatorInStation : IPlcBehaviorOperator
    {
        protected string plcAddrDesc1 = "AGV编号";
        protected string plcAddrDesc2 = "AGV编号";

        public virtual bool Begin(TaskData taskData, StepData stepData, out string msg)
        {
            var config = ExcelConfig.Instance.GetRouteConfig(stepData.WareHouseID);
            if (config == null)
            {
                msg = "查找库位配置信息失败";
                return false;
            }
            if (string.IsNullOrEmpty(config.PlcIpAddr))
            {
                msg = "PlcIp地址为空";
                return false;
            }

            var agvData = Crms.PmsApi.GetCarrier(taskData.Carrier);
            if (agvData == null)
            {
                msg = "获取AGV数据失败";
                return false;
            }

            var alarm = agvData.AlarmList?
                .Where(d => d.alarmLevel > 1)
                .OrderByDescending(d => d.alarmLevel)?
                .FirstOrDefault()?
                .alarmId ?? 0;

            var plcAddr = stepData.WareHouseID.GetLocationMember(plcAddrDesc1);
            var plcByteValues = new byte[] { (byte)taskData.Carrier, (byte)alarm };
            var result = config.PlcReadWrite.Write(plcAddr, plcByteValues);

            var plcByteValuesString = string.Join(",", plcByteValues);

            if (!result.IsSuccess)
            {
                msg = $"写入失败({plcAddrDesc1} {plcAddr} 写入 [{plcByteValuesString}]) Code:{result.ErrorCode} {result.Message}";
                return false;
            }

            msg = $"{plcAddr}开始已写[{plcByteValuesString}]";
            return true;
        }

        public virtual bool End(TaskData taskData, StepData stepData, out string msg)
        {
            var config = ExcelConfig.Instance.GetRouteConfig(stepData.WareHouseID);
            if (config == null)
            {
                msg = "查找库位配置信息失败";
                return false;
            }
            if (string.IsNullOrEmpty(config.PlcIpAddr))
            {
                msg = "PlcIp地址为空";
                return false;
            }

            var plcAddr = stepData.WareHouseID.GetLocationMember(plcAddrDesc2);
            var plcByteValues = new byte[] { 0, 0 };
            var result = config.PlcReadWrite.Write(plcAddr, plcByteValues);

            var plcByteValuesString = string.Join(",", plcByteValues);

            if (!result.IsSuccess)
            {
                msg = $"写入失败({plcAddrDesc2} {plcAddr} 写入 [{plcByteValuesString}]) Code:{result.ErrorCode} {result.Message}";
                return false;
            }

            msg = $"{plcAddr}结束已写[{plcByteValuesString}]";
            return true;
        }
    }


    public class PlcBehaviorOperator_Call : IPlcBehaviorOperator
    {
        protected string plcAddrDesc1 = "AGV反馈呼叫";
        protected string plcAddrDesc2 = "AGV反馈呼叫";

        public bool Begin(TaskData taskData, StepData stepData, out string msg)
        {
            var nextStepId = taskData.CurrentStepID + 1;
            var wareHouseID = taskData.StepList
                .FirstOrDefault(d => d.StepID == nextStepId)
                .WareHouseID;

            var config = ExcelConfig.Instance.GetRouteConfig(wareHouseID);
            if (config == null)
            {
                msg = "查找库位配置信息失败";
                return false;
            }
            if (string.IsNullOrEmpty(config.PlcIpAddr))
            {
                msg = "PlcIp地址为空";
                return false;
            }

            var plcAddr = wareHouseID.GetLocationMember(plcAddrDesc1);
            var plcValue = true;
            var result = config.PlcReadWrite.Write(plcAddr, plcValue);

            if (!result.IsSuccess)
            {
                msg = $"{wareHouseID} 写入失败({plcAddrDesc1} {plcAddr} 写入 [{plcValue}]) Code:{result.ErrorCode} {result.Message}";
                return false;
            }

            msg = $"{wareHouseID} {plcAddr}开始已写[{plcValue}]";
            return true;
        }

        public bool End(TaskData taskData, StepData stepData, out string msg)
        {
            msg = null;
            return true;
        }
    }
}
