﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskBookEvent
{
    public class TaskBookBehavior_63002 : BaseTaskBookBehaviorV2
    {
        LocationPropertyManager locationManager = new LocationPropertyManager();
        MaterialBindingManager materialBindingManager = new MaterialBindingManager();

        public TaskBookBehavior_63002() : base(63002, "库位取货")
        {
        }

        public override bool TaskBookOperator(TaskData taskDetailInfo, StepData stepInfo)
        {
            try
            {
                if (Crms.PmsTaskService is PmsTaskServiceSWms)
                {
                    return SWMS(taskDetailInfo, stepInfo);
                }
                else
                {
                    return WMS(taskDetailInfo, stepInfo);
                }
            }
            catch (Exception ex)
            {
                LogException(ex);
            }
            return false;
        }

        private bool SWMS(TaskData taskData, StepData stepData)
        {
            var agvStock = Crms.SWmsApi.GetStockByAgvId(taskData.Carrier, out var content);
            if (agvStock != null)
            {
                var result2 = Crms.SWmsApi.StockOutAutoByAgvId(taskData.Carrier, taskData.TaskID, out content);
                if (!result2)
                {
                    Log(taskData, stepData, $"清除AGV货物数据失败 {content}");
                    return false;
                }
            }

            var data = Crms.SWmsApi.GetLocationStockByLocationId(stepData.WareHouseID);
            if (data == null)
            {
                Log(taskData, stepData, $"获取库位信息失败");
                return false;
            }

            var result = Crms.SWmsApi.UpdateLocationLockMark(stepData.WareHouseID, data.Location.lockMark, null, out content);
            if (!result)
            {
                Log(taskData, stepData, $"任务占用解除失败 {(result ? null : content)}");
                return false;
            }

            result = Crms.SWmsApi.StockMoveAutoLocationToAgv(stepData.WareHouseID, taskData.Carrier, taskData.TaskID, out content);

            Log(taskData, stepData, $"{Remark}{result.ToChineseString()} {data?.Stock?.goodsCode} {data?.Stock?.goodsNumber} {(result ? null : content)}");
            return result;
        }

        private bool WMS(TaskData taskDetailInfo, StepData stepInfo)
        {
            var data = locationManager.GetById(stepInfo.WareHouseID);
            if (data == null)
            {
                Log(taskDetailInfo, stepInfo, $"获取库位信息失败");
                return false;
            }

            MaterialBinding materialBinding = new MaterialBinding();
            materialBinding.TaskID = taskDetailInfo.TaskID;
            materialBinding.CarrierID = taskDetailInfo.Carrier;

            materialBinding.MaterialBarcode = data.MaterialBarcode;
            materialBinding.MaterialID = data.MaterialId;
            materialBinding.MaterialName = data.MaterialName;
            materialBinding.MaterialNumber = data.MaterialNumber;
            materialBinding.MaterialWeight = data.MaterialWeight;
            materialBinding.BatchNumber = data.BatchNumber;
            materialBinding.ModifyTime = data.ModifyTime;

            materialBinding.TaskBook = taskDetailInfo.TaskBook;
            materialBinding.CreateTime = taskDetailInfo.CreateTime ?? default(DateTime);
            materialBinding.StartTime = taskDetailInfo.StartTime ?? default(DateTime);
            materialBinding.PickTime = DateTime.Now;
            materialBinding.StartWareHouse = data.WareHouseCode;
            materialBinding.StartLocation = data.LocationCode;

            if (!materialBindingManager.Save(materialBinding))
            {
                return false;
            }

            data.Status = LocationStatus.Empty;
            data.MaterialBarcode = string.Empty;
            data.MaterialId = string.Empty;
            data.MaterialName = string.Empty;
            data.MaterialNumber = string.Empty;
            data.MaterialWeight = string.Empty;
            data.BatchNumber = string.Empty;
            data.CurrentTaskID = string.Empty;

            data.ModifyTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            bool result = locationManager.Update(data);

            if (result)
            {
                //灭叫料盒灯
                int route = stepInfo.WareHouseID.GetLocationMember("放行任务路线").ToValue<int>();
                if (route > 0)
                {
                    for (int i = 0; i < 2; i++)
                    {
                        DeviceControl.Instance.CallBoxLightOff(route);
                        System.Threading.Thread.Sleep(100);
                    }
                }
            }

            Log(taskDetailInfo, stepInfo, $"库位取货，{materialBinding.MaterialID} 操作结果:{result}");
            return result;
        }
    }
}
