﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskBookEvent
{
    public class TaskBookBehavior_63003 : BaseTaskBookBehaviorV2
    {
        LocationPropertyManager locationManager = new LocationPropertyManager();
        MaterialBindingManager materialBindingManager = new MaterialBindingManager();

        public TaskBookBehavior_63003() : base(63003, "库位放货")
        {
        }

        public override bool TaskBookOperator(TaskData taskDetailInfo, StepData stepInfo)
        {
            try
            {
                if (Crms.PmsTaskService is PmsTaskServiceSWms)
                {
                    return SWMS(taskDetailInfo, stepInfo);
                }
                else
                {
                    return WMS(taskDetailInfo, stepInfo);
                }
            }
            catch (Exception ex)
            {
                LogException(ex);
            }
            return false;
        }

        private bool SWMS(TaskData taskData, StepData stepData)
        {
            var agvStock = Crms.SWmsApi.GetStockByAgvId(taskData.Carrier, out var content);
            if (agvStock == null)
            {
                Log(taskData, stepData, $"获取AGV货物数据失败 {content}");
                return false;
            }

            var data = Crms.SWmsApi.GetLocationStockByLocationId(stepData.WareHouseID);
            if (data == null)
            {
                Log(taskData, stepData, $"获取库位信息失败");
                return false;
            }

            var result = Crms.SWmsApi.UpdateLocationLockMark(stepData.WareHouseID, data.Location.lockMark, null, out content);
            if (!result)
            {
                Log(taskData, stepData, $"任务占用解除失败 {(result ? null : content)}");
                return false;
            }

            result = Crms.SWmsApi.StockMoveAutoAgvToLocation(taskData.Carrier, stepData.WareHouseID, taskData.TaskID, out content);
            data = Crms.SWmsApi.GetLocationStockByLocationId(stepData.WareHouseID);

            Log(taskData, stepData, $"{Remark}{result.ToChineseString()} {data?.Stock?.goodsCode} {data?.Stock?.goodsNumber} {(result ? null : content)}");
            return result;
        }

        private bool WMS(TaskData taskDetailInfo, StepData stepInfo)
        {
            var materialBinding = materialBindingManager.GetById(taskDetailInfo.TaskID);
            if (materialBinding == null)
            {
                Log(taskDetailInfo, stepInfo, $"获取任务物料绑定信息失败");
                return false;
            }

            var data = locationManager.GetById(stepInfo.WareHouseID);
            if (data == null)
            {
                Log(taskDetailInfo, stepInfo, $"获取库位信息失败");
                return false;
            }

            data.MaterialBarcode = materialBinding.MaterialBarcode;
            data.MaterialId = materialBinding.MaterialID;
            data.MaterialName = materialBinding.MaterialName;
            data.MaterialNumber = materialBinding.MaterialNumber;
            data.MaterialWeight = materialBinding.MaterialWeight;
            data.BatchNumber = materialBinding.BatchNumber;
            data.CurrentTaskID = string.Empty;

            data.Status = LocationStatus.Filled;
            data.ModifyTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
            bool result = locationManager.Update(data);
            if (!result)
            {
                Log(taskDetailInfo, stepInfo, $"更新库位数据失败");
                return false;
            }

            materialBinding.EndWareHouse = data.WareHouseCode;
            materialBinding.EndLocation = data.LocationCode;
            materialBinding.PlaceTime = DateTime.Now;
            result = materialBindingManager.Update(materialBinding);
            if (!result)
            {
                Log(taskDetailInfo, stepInfo, $"更新物料绑定数据失败");
                return false;
            }

            if (result)
            {
                //灭叫料盒灯
                int route = stepInfo.WareHouseID.GetLocationMember("叫料任务路线").ToValue<int>();
                if (route > 0)
                {
                    for (int i = 0; i < 2; i++)
                    {
                        DeviceControl.Instance.CallBoxLightOff(route);
                        System.Threading.Thread.Sleep(100);
                    }
                }
            }

            materialBindingManager.DeleteOldData(1);
            Log(taskDetailInfo, stepInfo, $"库位放货，{materialBinding.MaterialID} 操作结果:{result}");
            return result;
        }
    }
}
