﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskEvent
{
    public class ProcessAlarmReport : ICustomProcess
    {
        private int[] ignoreAlarmIds = ExcelConfig.Instance.VcuAlarmInfoList?.Where(d => d.Ignore).Select(d => d.AlarmId).ToArray();

        private Dictionary<int, ResultAgvData> agvDataDict = new Dictionary<int, ResultAgvData>();
        public string AllowInColumnName { get; set; } = "AGV滚筒已启动";

        public void CustomProcess(List<TaskData> taskDataList)
        {
            var agvDataList = Crms.PmsApi.GetAllCarrier();
            if (agvDataList == null) return;
            List<RouteConfig> routeConfigs = ExcelConfig.Instance.RouteConfigList.Where(p => p.WareHouseCode == "满箱上料" || p.WareHouseCode == "空箱上料").ToList();
            if (routeConfigs != null && routeConfigs.Count > 0)
            {
                short agvException = 0;
                bool agvAtStation = false;
                for (int i = 0; i < routeConfigs.Count; i++)
                {
                    var config = routeConfigs[i];
                    string allowInAddr = config.LocationCode.GetLocationMember(AllowInColumnName);
                    for (int j = 0; j < agvDataList.Count; j++)
                    {
                        var agvData = agvDataList[j];
                        var alarm = agvData.AlarmList?
                                .Where(d => d.alarmLevel > 1)?
                                .OrderByDescending(d => d.alarmLevel)?
                                .FirstOrDefault()?
                                .alarmId ?? 0;

                        if (agvData.GraphVertex == config.GraphVertex)
                        {
                            agvAtStation = true;
                            if (alarm > 0)
                            {
                                CLog.Instance.SystemLog.WriteInfo($" AGV{agvData.AgvID}# 报警代码:{alarm}#地图点:{config.GraphVertex}");
                                agvException = 1;
                            }

                            //bool allowIn = agvData.LogicBits.GetBitVaule(7);
                            //var result = config.PlcReadWrite.Write(allowInAddr, allowIn ? (short)1 : (short)0);
                           
                        }

                    }
                    if (!agvAtStation)
                    {
                        //config.PlcReadWrite.Write(allowInAddr, (short)0);
                        agvException = 1;
                    }
                    config.PlcReadWrite.WriteValue(config.LocationCode.GetLocationMember("AGV异常"), agvException);
                }
            }
        }

        public bool Report(List<TaskData> taskDataList, ResultAgvData agvData)
        {
            var alarm = agvData.AlarmList?
                .Where(d => d.alarmLevel > 1)?
                .OrderByDescending(d => d.alarmLevel)?
                .FirstOrDefault()?
                .alarmId ?? 0;

            if (ignoreAlarmIds?.Contains(alarm) == true) return true;

            //没有任务的车不上报
            var taskData = taskDataList.FirstOrDefault(d => d.Carrier == agvData.AgvID);
            //if (taskData == null) return true;

            var alarmInfo = ExcelConfig.Instance.GetVcuAlarmInfo(alarm);
            var alarmDescribe = alarmInfo == null ? "" : alarmInfo.AlarmName;

            var result = CustomerApi.AgvAlarm(alarmDescribe, agvData.AgvID.ToString(), taskData?.TaskID, out string content);
            if (!result)
            {
                CLog.Instance.SystemLog.WriteInfo($"{taskData?.TaskID} {agvData.AgvID}# {alarm} {alarmDescribe} 上报报警信息失败 {content}");
            }

            return result;
        }
    }
}

