﻿using DbCommon.BusinessCore.BaseCore;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskEvent
{
    public class ProcessCallBoxTask : ICustomProcess
    {
        private LocationPropertyManager locationManager = new LocationPropertyManager();

        private List<RouteConfig> configList = ExcelConfig.Instance.RouteConfigList
            .Where(d => d.WareHouseCode == "场景二叫料工位")
            .ToList();


        public void CustomProcess(List<TaskData> taskDataList)
        {
            CallTask();
            TransportTask();
        }



        private void CallTask()
        {
            for (int i = 0; i < configList.Count; i++)
            {
                var config = configList[i];

                int route = config.LocationCode.GetLocationMember("叫料任务路线").ToValue<int>();
                var dict = DeviceControl.Instance.Communication.InfoBoxDictionary;
                if (!dict.TryGetValue(route, out var info))
                {
                    continue;
                }

                if (!info.DataValid || !info.BoxCalled)
                {
                    continue;
                }

                var hasTask = Crms.PmsTaskService.LocationHasStepTask(config.LocationCode, out string _);
                if (hasTask)
                {
                    continue;
                }

                PmsTaskInfoDto data = new PmsTaskInfoDto();
                data.WareHouseCode = config.LocationCode.GetLocationMember("源仓库");
                data.LocationCode = config.LocationCode.GetLocationMember("源仓位");
                data.TargetLocationCode = config.LocationCode;

                var res = Crms.PmsTaskService.TaskAdd(data);

                var result = res.code == 20000;
                CLog.Instance.TaskLog.WriteInfo($"叫料盒添加叫料任务{result.ToChineseString()} {res.message}");
            }
        }

        private void TransportTask()
        {
            for (int i = 0; i < configList.Count; i++)
            {
                var config = configList[i];

                int route = config.LocationCode.GetLocationMember("放行任务路线").ToValue<int>();
                var dict = DeviceControl.Instance.Communication.InfoBoxDictionary;
                if (!dict.TryGetValue(route, out var info))
                {
                    continue;
                }

                if (!info.DataValid || !info.BoxCalled)
                {
                    continue;
                }

                var hasTask = Crms.PmsTaskService.LocationHasStepTask(config.LocationCode, out string _);
                if (hasTask)
                {
                    continue;
                }

                var locationData = locationManager.GetById(config.LocationCode);
                locationData.Status = DbCommon.Enties.DbModels.LocationStatus.Filled;
                locationManager.UpdateNotLocked(locationData);

                PmsTaskInfoDto data = new PmsTaskInfoDto();
                data.LocationCode = config.LocationCode;
                data.TargetWareHouseCode = config.LocationCode.GetLocationMember("目标仓库");
                data.TargetLocationCode = config.LocationCode.GetLocationMember("目标仓位");

                var res = Crms.PmsTaskService.TaskAdd(data);

                var result = res.code == 20000;
                CLog.Instance.TaskLog.WriteInfo($"叫料盒添加放行任务{result.ToChineseString()} {res.message}");
            }
        }
    }
}
