﻿using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models.Crms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskEvent
{
    public class ProcessPlcClear : ICustomProcess
    {
        private List<string> m_currentPosList = new List<string>();
        public void CustomProcess(List<TaskData> dataList)
        {
            m_currentPosList.Clear();
            for (int i = 0; i < dataList.Count; i++)
            {
                var data = dataList[i];

                var posArray = data.StepList?
                    .Where(d => !string.IsNullOrWhiteSpace(d.WareHouseID))?
                    .OrderBy(d => d.StepID)?
                    .Select(d => d.WareHouseID)?
                    .Distinct();

                if (posArray != null)
                {
                    m_currentPosList.AddRange(posArray);
                }
            }

            var configList = ExcelConfig.Instance.RouteConfigList;
            for (int i = 0; i < configList.Count; i++)
            {
                var config = configList[i];
                if (string.IsNullOrEmpty(config.PlcIpAddr) && config.IOTransID == 0)
                {
                    continue;
                }

                //工位没有任务了，需要清零信号
                if (!m_currentPosList.Contains(config.LocationCode))
                {
                    Clear(config);
                }
            }
        }


        private void Clear(RouteConfig config)
        {
            var plcBehaviorList = ExcelConfig.Instance.PlcBehaviorList
                .Where(d => !string.IsNullOrEmpty(d.Remark) && d.Remark.Contains("初始化"));

            if (plcBehaviorList == null
                || plcBehaviorList.Count() == 0)
            {
                return;
            }

            foreach (var item in plcBehaviorList)
            {
                foreach (string addrDesc in item.DbAddressWrite)
                {
                    string addr = config.LocationCode.GetPlcTaskAddr(addrDesc);
                    if (string.IsNullOrEmpty(addr)) continue;

                    int id = config.IOTransID;
                    if (id > 0)
                    {
                        var readResult2 = DeviceControl.Instance.GetOutputIO(id, addr);
                        if (readResult2.Item1
                            && readResult2.Item2)
                        {
                            DeviceControl.Instance.SetOutputIO(id, addr, false);
                            string strLog = $"{config.LocationCode} IO对接盒ID{id} 清零：{addr} ({addrDesc})";
                            CLog.Instance.SystemLog.WriteInfo(strLog);
                            Thread.Sleep(50);
                        }
                    }

                    switch (item.DbValueDataType)
                    {
                        case null:
                        case "":
                        case "bool":
                            Clear<bool>(config, addr, addrDesc);
                            break;
                        case "byte":
                            Clear<byte>(config, addr, addrDesc);
                            break;
                        case "short":
                        case "int16":
                            Clear<short>(config, addr, addrDesc);
                            break;
                        case "ushort":
                        case "uint16":
                            Clear<ushort>(config, addr, addrDesc);
                            break;
                        case "int":
                        case "int32":
                            Clear<int>(config, addr, addrDesc);
                            break;
                        case "uint":
                        case "uint32":
                            Clear<uint>(config, addr, addrDesc);
                            break;
                        case "long":
                        case "int64":
                            Clear<long>(config, addr, addrDesc);
                            break;
                        case "ulong":
                        case "uint64":
                            Clear<ulong>(config, addr, addrDesc);
                            break;
                        case "string":
                            Clear<string>(config, addr, addrDesc);
                            break;
                        default:
                            break;
                    }

                }
            }
        }

        private void Clear<T>(RouteConfig config, string addr, string addrDesc)
        {
            if (!config.PlcConnected || config.PlcReadWrite == null) return;

            var agvValue = default(T);
            var readResult = config.PlcReadWrite.ReadValue(addr, agvValue, out var _);
            if (readResult) return;

            var result = config.PlcReadWrite.WriteValue(addr, agvValue);
            CLog.Instance.TaskLog.WriteInfo($"{config.LocationCode} {config.PlcIpAddr} 清零：{addr} ({addrDesc}) 写入{result.IsSuccess.ToChineseString()} {(result.IsSuccess ? null : result.Message)}");
        }
    }
}
