﻿using DbCommon.BusinessCore.BaseCore;
using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Core;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.TaskEvent
{
    public class ProcessPlcTask : ICustomProcess
    {

        public void CustomProcess(List<TaskData> taskDataList)
        {
            UpdateLocationStock("缠膜机");

            CallTask("空栈板放置", "码垛机");

            TransportTask("码垛机", "缠膜机");
            TransportTask("缠膜机", "暂存内销");
        }

        private void CallTask(string startWarehouse, string endWarehouse)
        {
            var configList = ExcelConfig.Instance.RouteConfigList.Where(d => d.WareHouseCode == endWarehouse);

            foreach (var config in configList)
            {
                var plcCallAddr = config.LocationCode.GetLocationMember("PLC叫料");
                if (string.IsNullOrWhiteSpace(plcCallAddr))
                {
                    continue;
                }

                var readCall = config.PlcReadWrite.ReadBool(plcCallAddr);
                if (!readCall.IsSuccess
                    || readCall.Content == false)
                {
                    continue;
                }

                var startLocation = Crms.SWmsApi.GetLocationIdOut(startWarehouse);
                if (string.IsNullOrWhiteSpace(startLocation))
                {
                    continue;
                }

                var hasTask = Crms.PmsTaskService.LocationHasTask(config.LocationCode, out string _);
                if (hasTask)
                {
                    continue;
                }

                hasTask = Crms.PmsTaskService.LocationHasTask(startLocation, out string _);
                if (hasTask)
                {
                    continue;
                }

                Crms.PmsTaskService.UpdateLocationProperty(new LocationPropertyUpdateDto()
                {
                    LocationCode = config.LocationCode,
                    Status = LocationStatus.Empty
                });

                PmsTaskInfoDto dto = new PmsTaskInfoDto();
                dto.LocationCode = startLocation;
                dto.TargetLocationCode = config.LocationCode;

                var res = Crms.PmsTaskService.TaskAdd(dto);

                var result = res.code == 20000;
                CLog.Instance.TaskLog.WriteInfo($"PLC添加补料任务{result.ToChineseString()} {res.message} {config.LocationCode} {plcCallAddr}={readCall.Content}");
            }
        }


        private void TransportTask(string startWarehouse, string endWarehouse)
        {
            var configList = ExcelConfig.Instance.RouteConfigList.Where(d => d.WareHouseCode == startWarehouse);

            foreach (var config in configList)
            {
                var plcCallAddr = config.LocationCode.GetLocationMember("PLC转运");
                var plcForExportAddr = config.LocationCode.GetLocationMember("PLC外销");
                if (string.IsNullOrWhiteSpace(plcCallAddr)
                    || string.IsNullOrWhiteSpace(plcForExportAddr))
                {
                    continue;
                }

                var readCall = config.PlcReadWrite.ReadBool(plcCallAddr);
                if (!readCall.IsSuccess
                    || readCall.Content == false)
                {
                    continue;
                }

                var readExport = config.PlcReadWrite.ReadBool(plcForExportAddr);
                if (!readExport.IsSuccess)
                {
                    continue;
                }

                if (startWarehouse == "码垛机")
                {
                    endWarehouse = readExport.Content ? "人工打标" : "缠膜机";
                }

                if (startWarehouse == "缠膜机")
                {
                    endWarehouse = readExport.Content ? "暂存外销" : "暂存内销";
                }

                var tagetLocation = Crms.SWmsApi.GetLocationIdIn(endWarehouse);
                if (string.IsNullOrWhiteSpace(tagetLocation))
                {
                    continue;
                }

                var hasTask = Crms.PmsTaskService.LocationHasTask(config.LocationCode, out string _);
                if (hasTask)
                {
                    continue;
                }

                hasTask = Crms.PmsTaskService.LocationHasTask(tagetLocation, out string _);
                if (hasTask)
                {
                    continue;
                }

                Crms.PmsTaskService.UpdateLocationProperty(new LocationPropertyUpdateDto()
                {
                    LocationCode = config.LocationCode,
                    Status = LocationStatus.Filled
                });

                PmsTaskInfoDto dto = new PmsTaskInfoDto();
                dto.LocationCode = config.LocationCode;
                dto.TargetLocationCode = tagetLocation;

                var res = Crms.PmsTaskService.TaskAdd(dto);

                var result = res.code == 20000;
                CLog.Instance.TaskLog.WriteInfo($"PLC添加转运任务{result.ToChineseString()} {res.message} {config.LocationCode}=>{tagetLocation} {plcCallAddr}={readCall.Content};{plcForExportAddr}={readExport.Content}");
            }
        }

        private void UpdateLocationStock(string warehouse)
        {
            var configList = ExcelConfig.Instance.RouteConfigList.Where(d => d.WareHouseCode == warehouse);

            foreach (var config in configList)
            {
                var plcCallAddr = config.LocationCode.GetLocationMember("PLC叫料");
                var plcTransAddr = config.LocationCode.GetLocationMember("PLC转运");
                if (string.IsNullOrWhiteSpace(plcCallAddr)
                    || string.IsNullOrWhiteSpace(plcTransAddr))
                {
                    continue;
                }

                var readCall = config.PlcReadWrite.ReadBool(plcCallAddr);
                if (!readCall.IsSuccess)
                {
                    continue;
                }

                var readTrans = config.PlcReadWrite.ReadBool(plcTransAddr);
                if (!readTrans.IsSuccess)
                {
                    continue;
                }

                var hasTask = Crms.PmsTaskService.LocationHasStepTask(config.LocationCode, out string _);
                if (hasTask)
                {
                    continue;
                }

                var locationData = Crms.SWmsApi.GetLocationStockByLocationId(config.LocationCode);
                if (locationData == null
                    || !string.IsNullOrWhiteSpace(locationData.Location.lockMark))
                {
                    continue;
                }

                var status = LocationStatus.None;
                if (!readCall.Content && readTrans.Content)
                {
                    //满状态
                    status = LocationStatus.Filled;
                }
                else if(readCall.Content && !readTrans.Content)
                {
                    //空状态
                    status = LocationStatus.Empty;

                }

                //Crms.PmsTaskService.UpdateLocationProperty(new LocationPropertyUpdateDto()
                //{
                //    LocationCode = config.LocationCode,
                //    Status = status
                //});

                if ((status == LocationStatus.Filled && (locationData.Stock == null || locationData.Location.locationStatus == "0"))
                    || (status == LocationStatus.Empty && (locationData.Stock != null || locationData.Location.locationStatus == "0"))
                    || (status == LocationStatus.None && locationData.Location.locationStatus != "0"))
                {
                    var updateDto = new LocationPropertyUpdateDto()
                    {
                        LocationCode = config.LocationCode,
                        Status = status
                    };

                    var res = Crms.PmsTaskService.UpdateLocationProperty(updateDto);
                    var result = res?.code == 20000;
                    CLog.Instance.TaskLog.WriteInfo($"PLC更新缠膜机状态{updateDto.Status} {result.ToChineseString()} {res.message} {plcCallAddr}={readCall.Content};{plcTransAddr}={readTrans.Content}");
                }


                ////0：空 1：满
                //if ((readCall.Content && locationData.Stock == null)
                //    || (!readCall.Content && locationData.Stock != null))
                //{
                //    var updateDto = new LocationPropertyUpdateDto()
                //    {
                //        LocationCode = config.LocationCode,
                //        Status = readCall.Content ? LocationStatus.Filled : LocationStatus.Empty
                //    };

                //    var res = Crms.PmsTaskService.UpdateLocationProperty(updateDto);
                //    var result = res?.code == 20000;
                //    CLog.Instance.TaskLog.WriteInfo($"更新缠膜机状态{updateDto.Status} {result.ToChineseString()} {res.message} {plcCallAddr}={readCall.Content}");
                //}
            }
        }


    }
}
