﻿using ProjectManagementSystem.Device.Extenions;
using RRQMCore.ByteManager;
using RRQMSocket;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Device.Core
{
    public class CommunicationTcpClient : CommunicationBase
    {
        private string ipHost;
        private TcpClient tcpClient;

        public bool Online => tcpClient.Online;

        public CommunicationTcpClient(string ipAddress, int port)
        {
            ipHost = $"{ipAddress}:{port}";

            tcpClient = new TcpClient();
            tcpClient.Logger = new LogerBox(this.LogMsg);
            //tcpClient.UseReconnection(-1);

            //订阅事件
            tcpClient.Connected += (client, e) =>
            {
                LogMsg($"客户端已连接{ipHost} {e.Message}");
            };

            tcpClient.Disconnected += (client, e) =>
            {
                LogMsg($"客户端已断开{ipHost} {e.Message}");
            };

            tcpClient.Received += OnReceived;

            ////声明配置
            var config = new RRQMConfig();
            config.SetRemoteIPHost(new IPHost(ipHost));

            //载入配置
            tcpClient.Setup(config);
        }

        public void Connect()
        {
            tcpClient.Connect();
        }

        public void SendCommand(byte[] data)
        {
            try
            {
                if (tcpClient.Online)
                {
                    tcpClient.Send(data);
                    LogMsg($"{tcpClient.RemoteIPHost}: Send: {data.Byte2HexString()}");
                }
            }
            catch (Exception ex)
            {
                LogMsg($"{tcpClient.RemoteIPHost}: SendErr: {data.Byte2HexString()}", ex);
            }
        }

        private void OnReceived(TcpClient client, ByteBlock byteBlock, IRequestInfo obj)
        {
            byte[] data = new byte[byteBlock.Length];
            Buffer.BlockCopy(byteBlock.Buffer, 0, data, 0, data.Length);

            string clientName = client.MainSocket.RemoteEndPoint.ToString();
            LogMsg($"{clientName}: Recv: {data.Byte2HexString()}");

            DataReceived?.Invoke(clientName, data);
        }
    }
}
