﻿using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.WebApi.Extensions;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Web.Http.Controllers;
using System.Web.Http.Filters;

namespace ProjectManagementSystem.WebApi.Filters
{
    public class GlobalActionFilterAttribute : ActionFilterAttribute
    {
        private const string Key = "__action_duration__";

        public override void OnActionExecuting(HttpActionContext actionContext)
        {
            var stopWatch = new Stopwatch();
            actionContext.Request.Properties[Key] = stopWatch;
            stopWatch.Start();
        }

        public override void OnActionExecuted(HttpActionExecutedContext actionContext)
        {
            var stopWatch = actionContext.Request.Properties.TryGetValue(Key, out var prop) ? prop as Stopwatch : null;
            if (stopWatch != null)
            {
                stopWatch.Stop();
            }

            try
            {
                DateTime start = DateTime.Now;
                var requestContent = actionContext.Request?.Content?.ReadAsStringAsync().Result;
                var responseContent = actionContext.Response?.Content?.ReadAsStringAsync().Result;
                var clientIpAddress = actionContext.Request.GetClientIpAddress();
                var timeSpan = (DateTime.Now - start).TotalMilliseconds;

                string log = $"{clientIpAddress} {actionContext.Request.Method} {actionContext.Request.RequestUri} 【{stopWatch?.ElapsedMilliseconds}ms {timeSpan:F1}ms】{Environment.NewLine}{requestContent}{Environment.NewLine}{responseContent}";
                CLog.Instance.SystemLog.WirteDebugCaller(log);
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteExceptionCaller(ex);
            }
        }

    }
}
