﻿using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.WebApi.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http.Filters;

namespace ProjectManagementSystem.WebApi.Filters
{
    public class GlobalExceptionFilterAttribute : ExceptionFilterAttribute
    {
        public override void OnException(HttpActionExecutedContext actionContext)
        {
            CLog.Instance.SystemLog.WriteExceptionCaller(actionContext.Exception);

            try
            {
                DateTime start = DateTime.Now;

                var webMgrResponse = new { code = HttpStatusCode.InternalServerError, message = actionContext.Exception.Message };
                actionContext.Response = actionContext.Request.CreateResponse(HttpStatusCode.InternalServerError, webMgrResponse);

                var requestContent = actionContext.Request?.Content?.ReadAsStringAsync().Result;
                var responseContent = actionContext.Response?.Content?.ReadAsStringAsync().Result;
                var clientIpAddress = actionContext.Request.GetClientIpAddress();
                var timeSpan = (DateTime.Now - start).TotalMilliseconds;

                string log = $"{clientIpAddress} {actionContext.Request.Method} {actionContext.Request.RequestUri} 【{timeSpan:F1}ms】{Environment.NewLine}{requestContent}{Environment.NewLine}{responseContent}";
                CLog.Instance.SystemLog.WirteDebugCaller(log);
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteExceptionCaller(ex);
            }
        }
    }
}
