﻿using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.WebApi.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http.Controllers;
using System.Web.Http.Filters;

namespace ProjectManagementSystem.WebApi.Filters
{
    public class ValidateModelAttribute : ActionFilterAttribute
    {
        public override void OnActionExecuting(HttpActionContext actionContext)
        {
            if (actionContext.ModelState.IsValid) return;

            try
            {
                DateTime start = DateTime.Now;
                var webMgrResponse = new { code = HttpStatusCode.BadRequest, message = actionContext.ModelState.Values?.First()?.Errors[0]?.ErrorMessage };
                actionContext.Response = actionContext.Request.CreateResponse(HttpStatusCode.BadRequest, webMgrResponse);

                var requestContent = actionContext.Request?.Content?.ReadAsStringAsync().Result;
                var responseContent = actionContext.Response?.Content?.ReadAsStringAsync().Result;
                var clientIpAddress = actionContext.Request.GetClientIpAddress();
                var timeSpan = (DateTime.Now - start).TotalMilliseconds;

                string log = $"{clientIpAddress} {actionContext.Request.Method} {actionContext.Request.RequestUri} 【{timeSpan:F1}ms】{Environment.NewLine}{requestContent}{Environment.NewLine}{responseContent}";
                CLog.Instance.SystemLog.WirteDebugCaller(log);
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteExceptionCaller(ex);
            }

        }
    }
}
