﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectManagementSystem.UI
{
    public partial class DataGridUI : UserControl, IDisplayUI
    {
        private bool firstUpdate = true;
        private BindingSource bindingSource = new BindingSource();

        public DataGridUI()
        {
            InitializeComponent();
            DoubleBufferedDataGirdView(dataGridView1, true);

            dataGridView1.SelectionChanged += (s, e) =>
            {
                dataGridView1.AutoResizeColumns(DataGridViewAutoSizeColumnsMode.DisplayedCells);
            };

            this.dataGridView1.DataSource = bindingSource;
        }

        private void DoubleBufferedDataGirdView(DataGridView dgv, bool flag)
        {
            Type dgvType = dgv.GetType();
            PropertyInfo pi = dgvType.GetProperty("DoubleBuffered", BindingFlags.Instance | BindingFlags.NonPublic);
            pi.SetValue(dgv, flag, null);
        }


        public virtual bool Init()
        {
            return true;
        }

        public virtual void UpdateDisplay()
        {
        }

        public DataTable GetDataTable()
        {
            DataTable dt = new DataTable();
            for (int count = 0; count < dataGridView1.Columns.Count; count++)
            {
                if (dataGridView1.Columns[count].Visible)
                {
                    DataColumn dc = new DataColumn(dataGridView1.Columns[count].HeaderText.ToString());
                    dt.Columns.Add(dc);
                }
            }


            for (int count = 0; count < dataGridView1.Rows.Count; count++)
            {
                DataRow dr = dt.NewRow();

                int colCount = 0;
                for (int countsub = 0; countsub < dataGridView1.Columns.Count; countsub++)
                {
                    if (dataGridView1.Columns[countsub].Visible)
                    {
                        dr[colCount] = dataGridView1.Rows[count].Cells[countsub].Value.ToString();
                        colCount++;
                    }
                }
                dt.Rows.Add(dr);
            }

            return dt;
        }

        public string GetCurrentColValue(string colName)
        {
            if (dataGridView1.CurrentRow == null)
            {
                return null;
            }

            if (!dataGridView1.Columns.Contains(colName))
            {
                return null;
            }

            int DGVCurrentRow = dataGridView1.CurrentRow.Index;


            if (DGVCurrentRow < 0 || dataGridView1.Rows[DGVCurrentRow].Cells[colName].Value == null)
            {
                return null;
            }

            string strOrderID = dataGridView1.Rows[DGVCurrentRow].Cells[colName].Value.ToString();
            return strOrderID;
        }

        private void UpdateBindingSource<TS, TV>(IEnumerable<TS> dataList, Func<TS, TV, bool> predicate)
        {
            if (dataList == null || dataList.Count() == 0)
            {
                if (bindingSource.Count > 0)
                {
                    bindingSource.Clear();
                }

                return;
            }

            for (int i = bindingSource.Count - 1; i >= 0; i--)
            {
                var vm = bindingSource[i];

                if (vm is TV v)
                {
                    var data = dataList.FirstOrDefault(s => predicate(s, v));
                    if (data == null)
                    {
                        bindingSource.Remove(vm);
                    }
                }
            }

            foreach (var s in dataList)
            {
                TV vmData = default(TV);

                int idx = 0;
                for (int i = 0; i < bindingSource.Count; i++)
                {
                    var vm = bindingSource[i];
                    if (vm is TV v && predicate(s, v))
                    {
                        vmData = v;
                        idx = i;
                        break;
                    }
                }

                if (vmData == null)
                {
                    bindingSource.Add(s);
                }
                else
                {
                    bindingSource[idx] = s;
                }
            }
        }

        public void ShowDataGrid<T>(IEnumerable<T> dataSource, Func<T, T, bool> predicate = null)
        {
            this.Invoke(new MethodInvoker(() =>
            {
                this.SuspendLayout();

                if (predicate == null)
                {
                    bindingSource.DataSource = dataSource;
                }
                else
                {
                    UpdateBindingSource(dataSource, predicate);
                }

                foreach (DataGridViewColumn column in dataGridView1.Columns)
                {
                    if (column.Name.Contains("Time"))
                    {
                        column.DefaultCellStyle.Format = "G";
                    }
                    if (column.Name.Contains("HEX"))
                    {
                        column.DefaultCellStyle.Format = "X2";
                    }
                }

                var keyInfo = Common.Config.ExcelConfig.Instance.ColumnKeyInfoDict;
                foreach (DataGridViewColumn column in dataGridView1.Columns)
                {
                    if (keyInfo.TryGetValue(column.Name, out var columnKeyInfo))
                    {
                        column.HeaderText = columnKeyInfo.HeaderText;
                        column.Visible = columnKeyInfo.Visible;

                        if (!string.IsNullOrEmpty(columnKeyInfo.Format))
                        {
                            column.DefaultCellStyle.Format = columnKeyInfo.Format;
                        }
                    }
                    else
                    {
                        var displayName = typeof(T)?.GetProperty(column.Name)?.GetCustomAttribute<DisplayNameAttribute>()?.DisplayName;

                        if (!string.IsNullOrEmpty(displayName))
                        {
                            column.HeaderText = displayName;
                        }
                        //else
                        //{
                        //    column.Visible = false;
                        //}
                    }
                }


                if (firstUpdate && bindingSource.Count > 0)
                {
                    firstUpdate = false;
                    dataGridView1.AutoResizeColumns(DataGridViewAutoSizeColumnsMode.DisplayedCells);
                }

                this.ResumeLayout(false);
            }));

        }


    }
}
