﻿using DbCommon.BusinessCore.BaseCore;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Service;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectManagementSystem.UI
{
    public partial class DebugAreaTrafficUI : UserControl, IDisplayUI
    {
        private AreaTrafficManager areaTrafficManager = new AreaTrafficManager();
        public DebugAreaTrafficUI()
        {
            InitializeComponent();
        }

        public bool Init()
        {
            dataGridUI1.dataGridView1.CellClick += DataGridView1_CellClick;
            return true;
        }

        private void DataGridView1_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            textBox1.Text = dataGridUI1.GetCurrentColValue("Id");
            textBox2.Text = dataGridUI1.GetCurrentColValue("FactoryId");
            textBox3.Text = dataGridUI1.GetCurrentColValue("AgvId");
        }

        public void UpdateDisplay()
        {
            if (!this.Visible) return;

            var dataList = areaTrafficManager.GetList();
            dataGridUI1.ShowDataGrid(dataList, (s, v) => s.Id == v.Id);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                var senderText = (sender as Control).Text;

                string text = $"确定要{senderText} ？";
                var dialogResult = MessageBox.Show(text, "提示", MessageBoxButtons.OKCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
                if (dialogResult != DialogResult.OK)
                {
                    return;
                }

                AreaTrafficDto dto = new AreaTrafficDto();
                dto.Id = textBox1.Text;
                dto.FactoryId = textBox2.Text;
                dto.AgvId = textBox3.Text;

                var res = PmsCustomApi.Post<object>("AreaTrafficOut", dto);
                var result = res?.code == 20000;

                MessageBox.Show($"{senderText}: {res.message}", "提示", MessageBoxButtons.OK, result ? MessageBoxIcon.Information : MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteException("UI", ex);
                MessageBox.Show(ex.Message, "提示", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
