﻿using Microsoft.Xaml.Behaviors;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;

namespace ProjectManagementSystemView.Infrastructure.Behaviors
{
    public class MouseDragLocationBehavior : Behavior<FrameworkElement>
    {
        private Point pOrigin = new Point();
        private double pOriginX;
        private double pOriginY;

        private double CurX = 0;
        private double CurY = 0;
        private bool isDragging = false;
        private void Parent_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            var point = e.GetPosition(this.AssociatedObject);
            CurX = point.X;
            CurY = point.Y;

            pOriginX = pOrigin.X;
            pOriginY = pOrigin.Y;
            if (e.LeftButton == MouseButtonState.Pressed && Keyboard.IsKeyDown(Key.LeftCtrl))
            {
                isDragging = true;
                this.AssociatedObject.Cursor = Cursors.Hand;
                this.AssociatedObject.CaptureMouse();
                return;
            }
        }
        private void Parent_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (isDragging & e.LeftButton == MouseButtonState.Pressed)
            {
                var point = e.GetPosition(this.AssociatedObject);

                pOrigin.X = pOriginX + point.X - CurX;
                pOrigin.Y = pOriginY + point.Y - CurY;

                if (this.AssociatedObject.DataContext is Models.VmLocationProperty vm)
                {
                    vm.X += pOrigin.X;
                    vm.Y += pOrigin.Y;
                }
                return;
            }
        }
        private void Parent_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            this.AssociatedObject.ReleaseMouseCapture();
            this.AssociatedObject.Cursor = Cursors.Arrow;
            isDragging = false;
        }

        protected override void OnAttached()
        {
            this.AssociatedObject.MouseDown += Parent_MouseDown;
            this.AssociatedObject.MouseMove += Parent_MouseMove;
            this.AssociatedObject.MouseUp += Parent_MouseUp;
        }

        protected override void OnDetaching()
        {
            this.AssociatedObject.MouseDown -= Parent_MouseDown;
            this.AssociatedObject.MouseMove -= Parent_MouseMove;
            this.AssociatedObject.MouseUp -= Parent_MouseUp;
        }
    }
}
