﻿using Microsoft.Xaml.Behaviors;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;

namespace ProjectManagementSystemView.Infrastructure.Behaviors
{
    public class MouseTranslateZoomBehavior : Behavior<FrameworkElement>
    {
        private FrameworkElement parent;

        private Point pOrigin = new Point();
        private double pOriginX;
        private double pOriginY;

        private double resultScale = 1;
        private double mulScale = 0.9;

        private double CurX = 0;
        private double CurY = 0;
        private bool isDragging;
        private bool settingPosition;


        public double Scale
        {
            get { return (double)GetValue(ScaleProperty); }
            set { SetValue(ScaleProperty, value); }
        }
        public static readonly DependencyProperty ScaleProperty =
            DependencyProperty.Register("Scale", typeof(double), typeof(MouseTranslateZoomBehavior), new PropertyMetadata(1d, new PropertyChangedCallback(OnScaleChanged)));

        public double TranslateX
        {
            get { return (double)GetValue(TranslateXProperty); }
            set { SetValue(TranslateXProperty, value); }
        }
        public static readonly DependencyProperty TranslateXProperty =
            DependencyProperty.Register("TranslateX", typeof(double), typeof(MouseTranslateZoomBehavior), new PropertyMetadata(0d, new PropertyChangedCallback(OnTranslateXChanged)));

        public double TranslateY
        {
            get { return (double)GetValue(TranslateYProperty); }
            set { SetValue(TranslateYProperty, value); }
        }
        public static readonly DependencyProperty TranslateYProperty =
            DependencyProperty.Register("TranslateY", typeof(double), typeof(MouseTranslateZoomBehavior), new PropertyMetadata(0d, new PropertyChangedCallback(OnTranslateYChanged)));



        private static void OnScaleChanged(object sender, DependencyPropertyChangedEventArgs args)
        {
            var behavior = sender as MouseTranslateZoomBehavior;
            behavior?.UpdateTransformGraphic(behavior.TranslateX, behavior.TranslateY, (double)args.NewValue);
        }
        private static void OnTranslateXChanged(object sender, DependencyPropertyChangedEventArgs args)
        {
            var behavior = sender as MouseTranslateZoomBehavior;
            behavior?.UpdateTransformGraphic((double)args.NewValue, behavior.TranslateY, behavior.Scale);
        }
        private static void OnTranslateYChanged(object sender, DependencyPropertyChangedEventArgs args)
        {
            var behavior = sender as MouseTranslateZoomBehavior;
            behavior?.UpdateTransformGraphic(behavior.TranslateX, (double)args.NewValue, behavior.Scale);
        }


        private void Parent_MouseWheel(object sender, System.Windows.Input.MouseWheelEventArgs e)
        {
            double previewResultScale = resultScale;

            if (e.Delta > 0)
            {
                if (resultScale <= 9)
                {
                    resultScale /= mulScale;
                }
            }
            else
            {
                if (resultScale >= 0.1)
                {
                    resultScale *= mulScale;
                }
            }

            double canvasCenterX = pOrigin.X + parent.ActualWidth / 2;
            double canvasCenterY = pOrigin.Y + parent.ActualHeight / 2;
            Point mouseCenter = e.GetPosition(parent);

            double delX = mouseCenter.X - canvasCenterX;
            double delY = mouseCenter.Y - canvasCenterY;

            pOrigin.X = pOrigin.X - (resultScale - previewResultScale) * delX / previewResultScale;
            pOrigin.Y = pOrigin.Y - (resultScale - previewResultScale) * delY / previewResultScale;

            settingPosition = true;
            TransformGraphic(pOrigin.X, pOrigin.Y, resultScale, resultScale);
            settingPosition = false;
        }

        private void UpdateTransformGraphic(double xOffSet, double yOffSet, double scale)
        {
            if (settingPosition || AssociatedObject == null) return;

            resultScale = scale;
            pOrigin.X = xOffSet;
            pOrigin.Y = yOffSet;

            TransformGraphic(xOffSet, yOffSet, scale, scale);
        }

        private void TransformGraphic(double xOffSet, double yOffSet, double xScale, double yScale)
        {
            var group = this.AssociatedObject.RenderTransform as TransformGroup;
            if (group == null)
            {
                group = new TransformGroup();
                group.Children.Add(new ScaleTransform());
                group.Children.Add(new TranslateTransform());

                this.AssociatedObject.RenderTransform = group;
            }
            
            var scaleTransform = group.Children[0] as ScaleTransform;
            var translateTransform = group.Children[1] as TranslateTransform;

            scaleTransform.ScaleX = xScale;
            scaleTransform.ScaleY = yScale;

            translateTransform.X = xOffSet;
            translateTransform.Y = yOffSet;

            Scale = xScale;
            TranslateX = xOffSet;
            TranslateY = yOffSet;
        }

        private void Parent_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            var point = e.GetPosition(parent);
            CurX = point.X;
            CurY = point.Y;

            pOriginX = pOrigin.X;
            pOriginY = pOrigin.Y;
            if (e.MiddleButton == MouseButtonState.Pressed)
            {
                isDragging = true;
                this.AssociatedObject.Cursor = Cursors.Hand;
                return;
            }
        }
        private void Parent_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (isDragging & e.MiddleButton == MouseButtonState.Pressed)
            {
                var point = e.GetPosition(parent);

                pOrigin.X = pOriginX + point.X - CurX;
                pOrigin.Y = pOriginY + point.Y - CurY;

                settingPosition = true;
                TransformGraphic(pOrigin.X, pOrigin.Y, resultScale, resultScale);
                settingPosition = false;
            }
        }
        private void Parent_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            this.AssociatedObject.Cursor = Cursors.Arrow;
            isDragging = false;
        }

        protected override void OnAttached()
        {
            parent = this.AssociatedObject.Parent as FrameworkElement;
            if (parent != null)
            {
                parent.MouseDown += Parent_MouseDown;
                parent.MouseMove += Parent_MouseMove;
                parent.MouseUp += Parent_MouseUp;
                parent.MouseWheel += Parent_MouseWheel;
            }
        }

        protected override void OnDetaching()
        {
            if (parent != null)
            {
                parent.MouseDown -= Parent_MouseDown;
                parent.MouseMove -= Parent_MouseMove;
                parent.MouseUp -= Parent_MouseUp;
                parent.MouseWheel -= Parent_MouseWheel;
            }
        }
    }
}
