﻿using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace ProjectManagementSystemView.Views
{
    /// <summary>
    /// LogView.xaml 的交互逻辑
    /// </summary>
    public partial class LogView : UserControl
    {
        private int counter;
        private int logLevel;
        private DateTime lastScrollTime;
        private StringBuilder stringBuilder = new StringBuilder();
        private ConcurrentQueue<string> logQueue = new ConcurrentQueue<string>();

        public LogView()
        {
            InitializeComponent();

            logLevel = (int)comboBox1.SelectedItem;
            comboBox1.SelectionChanged += (s, e) =>
            {
                logLevel = (int)comboBox1.SelectedItem;
            };

            this.IsVisibleChanged += LogView_IsVisibleChanged;
            CLog.Instance.OnLog += Instance_OnLog;
        }

        private void LogView_IsVisibleChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            if (!IsVisible) return;

            try
            {
                if (logQueue.Count > 0)
                {
                    stringBuilder.Clear();
                    while (logQueue.TryDequeue(out var logText))
                    {
                        stringBuilder.AppendLine(logText);
                    }

                    textBox1.AppendText(stringBuilder.ToString());
                    if (checkBox1.IsChecked == true)
                    {
                        textBox1.ScrollToEnd();
                    }
                }
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteExceptionCaller(ex);
            }
        }

        private void Instance_OnLog(int degree, string log)
        {
            if (degree > logLevel) return;

            if (IsVisible)
            {
                this.Dispatcher.Invoke(() =>
                {
                    if (counter >= 200)
                    {
                        counter = 0;
                        textBox1.Clear();
                    }

                    if (logQueue.Count > 0)
                    {
                        stringBuilder.Clear();
                        while (logQueue.TryDequeue(out var logText))
                        {
                            stringBuilder.AppendLine(logText);
                        }

                        textBox1.AppendText(stringBuilder.ToString());
                    }

                    textBox1.AppendText($"{log}{Environment.NewLine}");
                    if (checkBox1.IsChecked == true && (DateTime.Now - lastScrollTime).TotalMilliseconds > 100)
                    {
                        textBox1.ScrollToEnd();
                        lastScrollTime = DateTime.Now;
                    }
                });
            }
            else
            {
                logQueue.Enqueue(log);
                if (logQueue.Count > 200)
                {
                    logQueue.TryDequeue(out var _);
                }
            }

            counter++;
        }
    }
}
